@@ -1,6 +1,6 @@
 /*
  * This is the gpsd driver for SiRF GPSes operating in binary mode.
- * It also handles uBlox, a SiRF derivative.
+ * It also handles early u-bloxes that were SiRF derivatives.
  *
  * The advantages: Reports climb/sink rate (raw-mode clients won't see this).
  * Also, we can flag DGPS satellites used in the skyview when SBAS is in use.
@@ -46,6 +46,12 @@
 #define HI(n)		((n) >> 8)
 #define LO(n)		((n) & 0xff)
 
+/*
+ * According to the protocol reference, if you don't get ACK/NACK in response
+ * to a control send withing 6 seconds, you should just retry.
+ */
+#define SIRF_RETRY_TIME	6
+
 /*@ +charint @*/
 /* Poll Software Version MID 132 */
 static unsigned char versionprobe[] = {
@@ -143,6 +149,7 @@ static unsigned char enablesubframe[] = {
     0x0C,			/* Chans 1-12 */
     /* change the next 0x10 to 0x08
      * for factory reset */
+    /* 0x10 turns on MIDs 7, 8, 17, 28, 29, 30 and 31 */
     0x10,
     0x00, 0x00, 0xb0, 0xb3
 };
@@ -159,7 +166,9 @@ static unsigned char disablesubframe[] = {
     0x00, 0x00,			/* week number */
     0x0C,			/* Chans 1-12 */
 
-    0x00,
+    /* 0x00 turns off MIDs 7, 8, 17, 28, 29, 30 and 31 */
+    0x00,                       /* reset bit map */
+                               
     0x00, 0x00, 0xb0, 0xb3
 };
 
@@ -195,8 +204,8 @@ static unsigned char enablemid52[] = {
 /*@ -charint @*/
 
 
-static gps_mask_t sirf_msg_debug(unsigned char *, size_t);
-static gps_mask_t sirf_msg_errors(unsigned char *, size_t);
+static gps_mask_t sirf_msg_debug(unsigned char *, size_t, const int);
+static gps_mask_t sirf_msg_errors(unsigned char *, size_t, const int);
 
 static gps_mask_t sirf_msg_navdata(struct gps_device_t *, unsigned char *,
 				   size_t);
@@ -206,6 +215,10 @@ static gps_mask_t sirf_msg_nlmd(struct gps_device_t *, unsigned char *,
 				size_t);
 static gps_mask_t sirf_msg_ppstime(struct gps_device_t *, unsigned char *,
 				   size_t);
+static gps_mask_t sirf_msg_nl(struct gps_device_t *, unsigned char *,
+				   size_t);
+static gps_mask_t sirf_msg_ee(struct gps_device_t *, unsigned char *,
+				   size_t);
 static gps_mask_t sirf_msg_svinfo(struct gps_device_t *, unsigned char *,
 				  size_t);
 static gps_mask_t sirf_msg_swversion(struct gps_device_t *, unsigned char *,
@@ -224,6 +237,22 @@ static bool sirf_write(struct gps_device_t *session, unsigned char *msg)
     size_t i, len;
     bool ok;
 
+    /*
+     * Control strings spaced too closely together confuse the SiRF
+     * IV.  This wasn't an issue on older SiRFs, but they've gone to a
+     * lower-powered processor that apparently has trouble keeping up.  
+     * Now you have to wait for the ACK, otherwise chaos ensues.
+     */
+    if (time(NULL) - session->driver.sirf.last_send > SIRF_RETRY_TIME)
+	session->driver.sirf.need_ack = false;
+    /* can also be false because ACK was received after last send */
+    if (session->driver.sirf.need_ack) {
+	gpsd_report(session->context->debug, LOG_WARN,
+		    "SiRF: write of control type %02x failed, awaiting ACK.\n",
+		    msg[4]);
+	return false;
+    }
+
     len = (size_t) ((msg[2] << 8) | msg[3]);
 
     /* calculate CRC */
@@ -236,8 +265,12 @@ static bool sirf_write(struct gps_device_t *session, unsigned char *msg)
     msg[len + 4] = (unsigned char)((crc & 0xff00) >> 8);
     msg[len + 5] = (unsigned char)(crc & 0x00ff);
 
-    gpsd_report(LOG_IO, "SiRF: Writing control type %02x:\n", msg[4]);
+    gpsd_report(session->context->debug, LOG_PROG,
+		"SiRF: Writing control type %02x:\n", msg[4]);
     ok = (gpsd_write(session, (const char *)msg, len+8) == (ssize_t) (len+8));
+ 
+    session->driver.sirf.need_ack = true;
+    session->driver.sirf.last_send = time(NULL);
     return (ok);
 }
 
@@ -281,10 +314,12 @@ static bool sirfbin_speed(struct gps_device_t *session, speed_t speed, char pari
 	0x00, 0x00, 0xb0, 0xb3
     };
     /*@ -charint @*/
-    gpsd_report(LOG_PROG, "SiRF: sirf_speed(%d,%c,%d)\n",
-		speed, parity, stopbits);
+    gpsd_report(session->context->debug, LOG_PROG,
+		"SiRF: sirf_speed(%u,%c,%d)\n",
+		(unsigned int)speed, parity, stopbits);
     if (9600 >= speed) {
-	gpsd_report(LOG_WARN, "SiRF may lag at 9600bps or less.\n");
+	gpsd_report(session->context->debug, LOG_WARN,
+		    "SiRF may lag at 9600bps or less.\n");
     }
 
     switch (parity) {
@@ -334,8 +369,9 @@ static bool sirf_to_nmea(struct gps_device_t *session, speed_t speed)
     /*@ -charint @*/
 
     if (speed >= 0xffff) {
-	gpsd_report(LOG_ERROR, "SiRF: can't switch from SiRF to NMEA because current speed %u is big.",
-		     speed);
+	gpsd_report(session->context->debug, LOG_ERROR,
+		    "SiRF: can't switch from SiRF to NMEA because current speed %u is big.",
+		     (unsigned int)speed);
 	return false;
     }
 
@@ -370,33 +406,13 @@ static void sirfbin_mode(struct gps_device_t *session, int mode)
 			session->gpsdata.dev.baudrate,
 			9 - session->gpsdata.dev.stopbits,
 			session->gpsdata.dev.stopbits, parity);
-	(void)usleep(333);	/* guessed settling time */
-	session->gpsdata.dev.driver_mode = MODE_BINARY;
     }
     session->back_to_nmea = false;
 }
 #endif /* RECONFIGURE_ENABLE */
 
-static ssize_t sirf_get(struct gps_device_t *session)
-{
-    ssize_t len = generic_get(session);
-
-    if (session->packet.type == SIRF_PACKET) {
-	session->gpsdata.dev.driver_mode = MODE_BINARY;
-    } else if (session->packet.type == NMEA_PACKET) {
-	session->gpsdata.dev.driver_mode = MODE_NMEA;
-	(void)gpsd_switch_driver(session, "Generic NMEA");
-    } else {
-	/* should never happen */
-	gpsd_report(LOG_PROG, "SiRF: Unexpected packet type %d\n",
-		    session->packet.type);
-	(void)gpsd_switch_driver(session, "Generic NMEA");
-    }
-
-    return len;
-}
-
-static gps_mask_t sirf_msg_debug(unsigned char *buf, size_t len)
+static gps_mask_t sirf_msg_debug(unsigned char *buf, size_t len,
+				 const int debug)
 {
     char msgbuf[MAX_PACKET_LENGTH * 3 + 2];
     int i;
@@ -409,7 +425,7 @@ static gps_mask_t sirf_msg_debug(unsigned char *buf, size_t len)
 	    (void)snprintf(msgbuf + strlen(msgbuf),
 			   sizeof(msgbuf) - strlen(msgbuf),
 			   "%c", buf[i] ^ 0xff);
-	gpsd_report(LOG_PROG, "SiRF: DEV 0xe1: %s\n", msgbuf);
+	gpsd_report(debug, LOG_PROG, "SiRF: DEV 0xe1: %s\n", msgbuf);
     } else if (0xff == (unsigned char)buf[0]) {	/* Debug messages */
 	for (i = 1; i < (int)len; i++)
 	    if (isprint(buf[i]))
@@ -419,28 +435,30 @@ static gps_mask_t sirf_msg_debug(unsigned char *buf, size_t len)
 		(void)snprintf(msgbuf + strlen(msgbuf),
 			       sizeof(msgbuf) - strlen(msgbuf),
 			       "\\x%02x", (unsigned int)buf[i]);
-	gpsd_report(LOG_PROG, "SiRF: DBG 0xff: %s\n", msgbuf);
+	gpsd_report(debug, LOG_PROG, "SiRF: DBG 0xff: %s\n", msgbuf);
     }
     /*@ -charint @*/
     return 0;
 }
 
-static gps_mask_t sirf_msg_errors(unsigned char *buf, size_t len UNUSED)
+static gps_mask_t sirf_msg_errors(unsigned char *buf, size_t len UNUSED,
+				  const int debug)
 {
     switch (getbeu16(buf, 1)) {
     case 2:
-	gpsd_report(LOG_PROG,
+	gpsd_report(debug, LOG_PROG,
 		    "SiRF: EID 0x0a type 2: Subframe %u error on PRN %u\n",
 		    getbeu32(buf, 9), getbeu32(buf, 5));
 	break;
 
     case 4107:
-	gpsd_report(LOG_PROG,
+	gpsd_report(debug, LOG_PROG,
 		    "SiRF: EID 0x0a type 4107: neither KF nor LSQ fix.\n");
 	break;
 
     default:
-	gpsd_report(LOG_PROG, "SiRF: EID 0x0a: Error MID %d\n",
+	gpsd_report(debug, LOG_PROG,
+		    "SiRF: EID 0x0a: Error MID %d\n",
 		    getbeu16(buf, 1));
 	break;
     }
@@ -448,7 +466,7 @@ static gps_mask_t sirf_msg_errors(unsigned char *buf, size_t len UNUSED)
 }
 
 /* Navigation Library Measurement Data MID 28 */
-static gps_mask_t sirf_msg_nlmd(struct gps_device_t *session UNUSED,
+static gps_mask_t sirf_msg_nlmd(struct gps_device_t *session,
 				unsigned char *buf UNUSED, size_t len)
 {
 
@@ -461,7 +479,8 @@ static gps_mask_t sirf_msg_nlmd(struct gps_device_t *session UNUSED,
      * different double orders, neither IEEE754 */
     /* FIXME - decode the time, since this is the first MID with a
      * good time stamp this will be good for ntpshm time */
-    gpsd_report(LOG_PROG, "SiRF: MID 0x1c, NLMD, gps_tow: %f\n",
+    gpsd_report(session->context->debug, LOG_PROG,
+		"SiRF: MID 0x1c, NLMD, gps_tow: %f\n",
 		(double)gps_tow);
 
     return 0;
@@ -493,13 +512,16 @@ static gps_mask_t sirf_msg_swversion(struct gps_device_t *session,
 	session->driver.sirf.driverstate |= SIRF_GE_232;
     }
     if (strstr((char *)(buf + 1), "ES"))
-	gpsd_report(LOG_INF, "SiRF: Firmware has XTrac capability\n");
-    gpsd_report(LOG_PROG, "SiRF: fv: %0.2f, Driver state flags are: %0x\n",
+	gpsd_report(session->context->debug, LOG_INF,
+		    "SiRF: Firmware has XTrac capability\n");
+    gpsd_report(session->context->debug, LOG_PROG,
+		"SiRF: fv: %0.2f, Driver state flags are: %0x\n",
 		fv, session->driver.sirf.driverstate);
-#ifdef NTPSHM_ENABLE
+#ifdef TIMEHINT_ENABLE
     session->driver.sirf.time_seen = 0;
-#endif /* NTPSHM_ENABLE */
-    gpsd_report(LOG_DATA, "SiRF: FV MID 0x06: subtype='%s' mask={DEVICEID}\n",
+#endif /* TIMEHINT_ENABLE */
+    gpsd_report(session->context->debug, LOG_DATA,
+		"SiRF: FV MID 0x06: subtype='%s' mask={DEVICEID}\n",
 		session->subtype);
     return DEVICEID_SET;
 }
@@ -520,12 +542,14 @@ static gps_mask_t sirf_msg_navdata(struct gps_device_t *session,
 	words[i] = (uint32_t)getbeu32(buf, 4 * i + 3);
     }
 
-    gpsd_report(LOG_IO, "SiRF: NavData chan %u svid %u\n",chan,svid);
+    gpsd_report(session->context->debug, LOG_DATA,
+		"SiRF: NavData chan %u svid %u\n",chan,svid);
 
 #ifdef RECONFIGURE_ENABLE
-    if (!session->context->readonly && session->gpsdata.dev.baudrate < 38400) {
+    /* SiRF recommends at least 57600 for SiRF IV nav data */
+    if (!session->context->readonly && session->gpsdata.dev.baudrate < 57600) {
         /* some USB are also too slow, no way to tell which ones */
-	gpsd_report(LOG_WARN,
+	gpsd_report(session->context->debug, LOG_WARN,
 		"WARNING: SiRF: link too slow, disabling subframes.\n");
 	(void)sirf_write(session, disablesubframe);
     }
@@ -539,7 +563,7 @@ static gps_mask_t sirf_msg_navdata(struct gps_device_t *session,
 static gps_mask_t sirf_msg_svinfo(struct gps_device_t *session,
 				  unsigned char *buf, size_t len)
 {
-    int st, i, j, cn;
+    int st, i, j;
 
     if (len != 188)
 	return 0;
@@ -550,6 +574,7 @@ static gps_mask_t sirf_msg_svinfo(struct gps_device_t *session,
 
     gpsd_zero_satellites(&session->gpsdata);
     for (i = st = 0; i < SIRF_CHANNELS; i++) {
+	int cn;
 	int off = 8 + 15 * i;
 	bool good;
 	session->gpsdata.PRN[st] = (int)getub(buf, off);
@@ -566,7 +591,7 @@ static gps_mask_t sirf_msg_svinfo(struct gps_device_t *session,
 	    session->gpsdata.azimuth[st] != 0 &&
 	    session->gpsdata.elevation[st] != 0;
 #ifdef __UNUSED__
-	gpsd_report(LOG_PROG,
+	gpsd_report(session->context->debug, LOG_PROG,
 		    "SiRF: PRN=%2d El=%3.2f Az=%3.2f ss=%3d stat=%04x %c\n",
 		    getub(buf, off),
 		    getub(buf, off + 2) / 2.0,
@@ -585,26 +610,27 @@ static gps_mask_t sirf_msg_svinfo(struct gps_device_t *session,
 		&& session->driver.sirf.dgps_source == SIRF_DGPS_SOURCE_SBAS)
 	    session->gpsdata.used[session->gpsdata.satellites_used++] = prn;
     }
-#ifdef NTPSHM_ENABLE
+#ifdef TIMEHINT_ENABLE
     if (st < 3) {
-	gpsd_report(LOG_PROG,
+	gpsd_report(session->context->debug, LOG_PROG,
 		    "SiRF: NTPD not enough satellites seen: %d\n", st);
     } else {
 	/* SiRF says if 3 sats in view the time is good */
-	gpsd_report(LOG_PROG,
+	gpsd_report(session->context->debug, LOG_PROG,
 		    "SiRF: NTPD valid time MID 0x04, seen=0x%02x, time:%.2lf, leap:%d\n",
 		    session->driver.sirf.time_seen,
 		    session->gpsdata.skyview_time,
 		    session->context->leap_seconds);
     }
-#endif /* NTPSHM_ENABLE */
-    gpsd_report(LOG_DATA, "SiRF: MTD 0x04: visible=%d mask={SATELLITE}\n",
+#endif /* TIMEHINT_ENABLE */
+    gpsd_report(session->context->debug, LOG_DATA,
+		"SiRF: MTD 0x04: visible=%d mask={SATELLITE}\n",
 		session->gpsdata.satellites_visible);
     return SATELLITE_SET;
 }
 
-#ifdef NTPSHM_ENABLE
-static double sirf_ntp_offset(struct gps_device_t *session)
+#ifdef TIMEHINT_ENABLE
+static double sirf_time_offset(struct gps_device_t *session)
 /* return NTP time-offset fudge factor for this device */
 {
     double retval = NAN;
@@ -619,7 +645,7 @@ static double sirf_ntp_offset(struct gps_device_t *session)
 	retval = 0.3;
     }
 
-    /* uBlox EMND message */
+    /* u-blox EMND message */
     else if (strcmp(session->gpsdata.tag, "MID98") == 0) {
 	retval = 0.570;
     }
@@ -657,7 +683,7 @@ static double sirf_ntp_offset(struct gps_device_t *session)
 
     return retval;
 }
-#endif /* NTPSHM_ENABLE */
+#endif /* TIMEHINT_ENABLE */
 
 static gps_mask_t sirf_msg_navsol(struct gps_device_t *session,
 				  unsigned char *buf, size_t len)
@@ -695,24 +721,25 @@ static gps_mask_t sirf_msg_navsol(struct gps_device_t *session,
 	session->newdata.mode = MODE_2D;
     if (session->newdata.mode == MODE_3D)
 	mask |= ALTITUDE_SET | CLIMB_SET;
-    gpsd_report(LOG_PROG,
+    gpsd_report(session->context->debug, LOG_PROG,
 		"SiRF: MND 0x02: Navtype = 0x%0x, Status = %d, mode = %d\n",
 		navtype, session->gpsdata.status, session->newdata.mode);
     /* byte 20 is HDOP, see below */
     /* byte 21 is "mode 2", not clear how to interpret that */
     session->newdata.time = gpsd_gpstime_resolve(session,
 	(unsigned short)getbes16(buf, 22), (double)getbeu32(buf, 24) * 1e-2);
-#ifdef NTPSHM_ENABLE
+#ifdef TIMEHINT_ENABLE
     if (session->newdata.mode <= MODE_NO_FIX) {
-	gpsd_report(LOG_PROG, "SiRF: NTPD no fix, mode: %d\n",
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "SiRF: NTPD no fix, mode: %d\n",
 		    session->newdata.mode);
     } else {
-	gpsd_report(LOG_PROG,
+	gpsd_report(session->context->debug, LOG_PROG,
 		    "SiRF: NTPD valid time MID 0x02, seen=0x%02x, time;%.2lf, leap:%d\n",
 		    session->driver.sirf.time_seen,
 		    session->newdata.time, session->context->leap_seconds);
     }
-#endif /* NTPSHM_ENABLE */
+#endif /* TIMEHINT_ENABLE */
     /* fix quality data */
     session->gpsdata.dop.hdop = (double)getub(buf, 20) / 5.0;
     mask |=
@@ -721,7 +748,7 @@ static gps_mask_t sirf_msg_navsol(struct gps_device_t *session,
     if ( 3 <= session->gpsdata.satellites_visible ) {
 	mask |= PPSTIME_IS;
     }
-    gpsd_report(LOG_DATA,
+    gpsd_report(session->context->debug, LOG_DATA,
 		"SiRF: MND 0x02: time=%.2f lat=%.2f lon=%.2f alt=%.2f track=%.2f speed=%.2f mode=%d status=%d hdop=%.2f used=%d\n",
 		session->newdata.time, session->newdata.latitude,
 		session->newdata.longitude, session->newdata.altitude,
@@ -781,7 +808,7 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
 	session->newdata.mode = MODE_3D;
     else if (session->gpsdata.status)
 	session->newdata.mode = MODE_2D;
-    gpsd_report(LOG_PROG,
+    gpsd_report(session->context->debug, LOG_PROG,
 		"SiRF: GND 0x29: Navtype = 0x%0x, Status = %d, mode = %d\n",
 		navtype, session->gpsdata.status, session->newdata.mode);
     mask |= STATUS_SET | MODE_SET;
@@ -849,17 +876,20 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
 	/*@ -compdef -unrecog */
 	session->newdata.time = (timestamp_t)timegm(&unpacked_date) + subseconds;
 	/*@ +compdef +unrecog */
-	gpsd_report(LOG_PROG, "SiRF: GND 0x29 UTC: %lf\n",
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "SiRF: GND 0x29 UTC: %lf\n",
 		    session->newdata.time);
-#ifdef NTPSHM_ENABLE
+#ifdef TIMEHINT_ENABLE
 	if (session->newdata.mode <= MODE_NO_FIX) {
-	    gpsd_report(LOG_PROG, "SiRF: NTPD no fix, mode: $d\n",
+	    gpsd_report(session->context->debug, LOG_PROG,
+			"SiRF: NTPD no fix, mode: $d\n",
 			session->newdata.mode);
 	} else if (0 == unpacked_date.tm_year) {
-	    gpsd_report(LOG_PROG, "SiRF: NTPD no year\n",
+	    gpsd_report(session->context->debug, LOG_PROG,
+			"SiRF: NTPD no year\n",
 			session->newdata.mode);
 	} else {
-	    gpsd_report(LOG_PROG,
+	    gpsd_report(session->context->debug, LOG_PROG,
 			"SiRF: NTPD valid time MID 0x29, seen=0x%02x\n",
 			session->driver.sirf.time_seen);
 	}
@@ -867,7 +897,7 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
 	    mask |= PPSTIME_IS;
 	}
 
-#endif /* NTPSHM_ENABLE */
+#endif /* TIMEHINT_ENABLE */
 	/* skip 4 bytes of satellite map */
 	session->newdata.altitude = getbes32(buf, 35) * 1e-2;
 	/* skip 1 byte of map datum */
@@ -879,7 +909,7 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
 	if (session->newdata.mode == MODE_3D)
 	    mask |= ALTITUDE_SET | CLIMB_SET;
     }
-    gpsd_report(LOG_DATA,
+    gpsd_report(session->context->debug, LOG_DATA,
 		"SiRF: GND 0x29: time=%.2f lat=%.2f lon=%.2f alt=%.2f track=%.2f speed=%.2f mode=%d status=%d\n",
 		session->newdata.time,
 		session->newdata.latitude,
@@ -911,7 +941,8 @@ static gps_mask_t sirf_msg_sysparam(struct gps_device_t *session,
     session->driver.sirf.track_smooth_mode = (unsigned char)getub(buf, 12);
 #ifdef RECONFIGURE_ENABLE
     if (!session->context->readonly) {
-	gpsd_report(LOG_PROG, "SiRF: Setting Navigation Parameters\n");
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "SiRF: Setting Navigation Parameters\n");
 	(void)sirf_write(session, modecontrol);
     }
 #endif /* RECONFIGURE_ENABLE */
@@ -935,7 +966,7 @@ static gps_mask_t sirf_msg_ublox(struct gps_device_t *session,
     if (len != 39)
 	return 0;
 
-    /* this packet is only sent by uBlox firmware from version 1.32 */
+    /* this packet is only sent by u-blox firmware from version 1.32 */
     mask = LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET |
 	STATUS_SET | MODE_SET | DOP_SET;
     session->newdata.latitude = (double)getbes32(buf, 1) * RAD_2_DEG * 1e-8;
@@ -960,7 +991,7 @@ static gps_mask_t sirf_msg_ublox(struct gps_device_t *session,
 	session->newdata.mode = MODE_3D;
     else if (session->gpsdata.status)
 	session->newdata.mode = MODE_2D;
-    gpsd_report(LOG_PROG,
+    gpsd_report(session->context->debug, LOG_PROG,
 		"SiRF: EMND 0x62: Navtype = 0x%0x, Status = %d, mode = %d\n",
 		navtype, session->gpsdata.status, session->newdata.mode);
 
@@ -981,15 +1012,16 @@ static gps_mask_t sirf_msg_ublox(struct gps_device_t *session,
 	/*@ -compdef */
 	session->newdata.time = (timestamp_t)mkgmtime(&unpacked_date) + subseconds;
 	/*@ +compdef */
-#ifdef NTPSHM_ENABLE
+#ifdef TIMEHINT_ENABLE
 	if (0 == (session->driver.sirf.time_seen & TIME_SEEN_UTC_2)) {
-	    gpsd_report(LOG_RAW, "SiRF: NTPD just SEEN_UTC_2\n");
+	    gpsd_report(session->context->debug, LOG_RAW,
+			"SiRF: NTPD just SEEN_UTC_2\n");
 	}
-	gpsd_report(LOG_PROG,
+	gpsd_report(session->context->debug, LOG_PROG,
 		    "SiRF: NTPD valid time MID 0x62, seen=0x%02x\n",
 		    session->driver.sirf.time_seen);
 	session->driver.sirf.time_seen |= TIME_SEEN_UTC_2;
-#endif /* NTPSHM_ENABLE */
+#endif /* TIMEHINT_ENABLE */
 	session->context->valid |= LEAP_SECOND_VALID;
     }
 
@@ -999,7 +1031,7 @@ static gps_mask_t sirf_msg_ublox(struct gps_device_t *session,
     session->gpsdata.dop.vdop = (int)getub(buf, 37) / 5.0;
     session->gpsdata.dop.tdop = (int)getub(buf, 38) / 5.0;
     session->driver.sirf.driverstate |= UBLOX;
-    gpsd_report(LOG_DATA,
+    gpsd_report(session->context->debug, LOG_DATA,
 		"SiRF: EMD 0x62: time=%.2f lat=%.2f lon=%.2f alt=%.f speed=%.2f track=%.2f climb=%.2f mode=%d status=%d gdop=%.2f pdop=%.2f hdop=%.2f vdop=%.2f tdop=%.2f\n",
 		session->newdata.time, session->newdata.latitude,
 		session->newdata.longitude, session->newdata.altitude,
@@ -1019,7 +1051,8 @@ static gps_mask_t sirf_msg_ppstime(struct gps_device_t *session,
     if (len != 19)
 	return 0;
 
-    gpsd_report(LOG_PROG, "SiRF: PPS 0x34: Status = 0x%02x\n",
+    gpsd_report(session->context->debug, LOG_PROG,
+		"SiRF: PPS 0x34: Status = 0x%02x\n",
 		getub(buf, 14));
     if (((int)getub(buf, 14) & 0x07) == 0x07) {	/* valid UTC time? */
 	struct tm unpacked_date;
@@ -1034,16 +1067,17 @@ static gps_mask_t sirf_msg_ppstime(struct gps_device_t *session,
 	/*@ +compdef */
 	session->context->leap_seconds = (int)getbeu16(buf, 8);
 	session->context->valid |= LEAP_SECOND_VALID;
-#ifdef NTPSHM_ENABLE
+#ifdef TIMEHINT_ENABLE
 	if (0 == (session->driver.sirf.time_seen & TIME_SEEN_UTC_2)) {
-	    gpsd_report(LOG_RAW, "SiRF: NTPD just SEEN_UTC_2\n");
+	    gpsd_report(session->context->debug, LOG_RAW,
+			"SiRF: NTPD just SEEN_UTC_2\n");
 	}
-	gpsd_report(LOG_PROG,
+	gpsd_report(session->context->debug, LOG_PROG,
 		    "SiRF: NTPD valid time MID 0x34, seen=0x%02x, leap=%d\n",
 		    session->driver.sirf.time_seen,
 		    session->context->leap_seconds);
 	session->driver.sirf.time_seen |= TIME_SEEN_UTC_2;
-#endif /* NTPSHM_ENABLE */
+#endif /* TIMEHINT_ENABLE */
 	mask |= TIME_SET;
 	if ( 3 <= session->gpsdata.satellites_visible ) {
 	    mask |= PPSTIME_IS;
@@ -1052,6 +1086,90 @@ static gps_mask_t sirf_msg_ppstime(struct gps_device_t *session,
     return mask;
 }
 
+static gps_mask_t sirf_msg_nl(struct gps_device_t *session,
+				   unsigned char *buf, size_t len)
+{
+
+    if (len != 67)
+	return 0;
+
+    switch ( buf[1] ) {
+    case 1:
+	gpsd_report(session->context->debug, LOG_PROG, 
+		    "SiRF IV: unused EE 0x38, SubID: 1, GPS Data\n");
+	break;
+    case 2:
+	gpsd_report(session->context->debug, LOG_PROG, 
+	    "SiRF IV: unused EE 0x38, SubID: 2, EE Integrity\n");
+	break;
+    case 3:
+	gpsd_report(session->context->debug, LOG_PROG, 
+	    "SiRF IV: unused EE 0x38, SubID: 3, EE Integrity\n");
+	break;
+    case 4:
+	gpsd_report(session->context->debug, LOG_PROG, 
+	    "SiRF IV: unused EE 0x38, SubID: 4, EE Clock Bias\n");
+	break;
+    case 5:
+	gpsd_report(session->context->debug, LOG_PROG, 
+	    "SiRF IV: unused EE 0x38, SubID: 4, 50bps\n");
+	break;
+    case 32:
+	gpsd_report(session->context->debug, LOG_PROG, 
+	    "SiRF IV: unused EE 0x38, SubID: 4, ECLM ACK/NACK\n");
+	break;
+    case 33:
+	gpsd_report(session->context->debug, LOG_PROG, 
+	    "SiRF IV: unused EE 0x38, SubID: 4, ECLM EE Age\n");
+	break;
+    case 34:
+	gpsd_report(session->context->debug, LOG_PROG, 
+	    "SiRF IV: unused EE 0x38, SubID: 4, ECLM SGEE Age\n");
+	break;
+    case 35:
+	gpsd_report(session->context->debug, LOG_PROG, 
+	    "SiRF IV: unused EE 0x38, SubID: 4, ECLM Download Intiate\n");
+	break;
+    case 255:
+	gpsd_report(session->context->debug, LOG_PROG, 
+	    "SiRF IV: unused EE 0x38, SubID: 4, EE ACK\n");
+	break;
+    default:
+	gpsd_report(session->context->debug, LOG_PROG, 
+		    "SiRF IV: unused EE 0x38, unknown SubID: %d\n",
+		    buf[1]);
+    }
+
+    return 0;
+}
+
+static gps_mask_t sirf_msg_ee(struct gps_device_t *session,
+				   unsigned char *buf, size_t len)
+{
+
+    if (len != 67)
+	return 0;
+
+    switch ( buf[1] ) {
+    case 1:
+	gpsd_report(session->context->debug, LOG_PROG, 
+		    "SiRF IV: unused NL 0x40, SubID: 1\n");
+	break;
+    case 2:
+	gpsd_report(session->context->debug, LOG_PROG, 
+		    "SiRF IV: unused NL 0x40, SubID: 2, PRN: %d\n",
+		    buf[2]);
+	break;
+    default:
+	gpsd_report(session->context->debug, LOG_PROG, 
+		    "SiRF IV: unused NL 0x40, unknown SubID: %d\n",
+		    buf[1]);
+    }
+
+    return 0;
+}
+
+
 gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 		      size_t len)
 {
@@ -1061,7 +1179,8 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 
     buf += 4;
     len -= 8;
-    gpsd_report(LOG_RAW, "SiRF: Raw packet type 0x%02x\n", buf[0]);
+    gpsd_report(session->context->debug, LOG_RAW,
+		"SiRF: Raw packet type 0x%02x\n", buf[0]);
     (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag),
 		   "MID%d", (int)buf[0]);
 
@@ -1074,22 +1193,23 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 	    return sirf_msg_navsol(session, buf,
 				   len) | (CLEAR_IS | REPORT_IS);
 	else {
-	    gpsd_report(LOG_PROG,
-			"SiRF: MND 0x02 skipped, uBlox flag is on.\n");
+	    gpsd_report(session->context->debug, LOG_PROG,
+			"SiRF: MND 0x02 skipped, u-blox flag is on.\n");
 	    return 0;
 	}
     case 0x04:			/* Measured tracker data out MID 4 */
 	return sirf_msg_svinfo(session, buf, len);
 
     case 0x05:			/* Raw Tracker Data Out MID 5 */
-	gpsd_report(LOG_PROG, "SiRF: unused Raw Tracker Data 0x05\n");
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "SiRF: unused Raw Tracker Data 0x05\n");
 	return 0;
 
     case 0x06:			/* Software Version String MID 6 */
 	return sirf_msg_swversion(session, buf, len);
 
     case 0x07:			/* Clock Status Data MID 7 */
-	gpsd_report(LOG_PROG, "SiRF: unused CLK 0x07\n");
+	gpsd_report(session->context->debug, LOG_PROG, "SiRF: unused CLK 0x07\n");
 	return 0;
 
     case 0x08:			/* subframe data MID 8 */
@@ -1107,7 +1227,7 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 	return sirf_msg_navdata(session, buf, len);
 
     case 0x09:			/* CPU Throughput MID 9 */
-	gpsd_report(LOG_PROG,
+	gpsd_report(session->context->debug, LOG_PROG,
 		    "SiRF: THR 0x09: SegStatMax=%.3f, SegStatLat=%3.f, AveTrkTime=%.3f, Last MS=%u\n",
 		    (float)getbeu16(buf, 1) / 186, (float)getbeu16(buf,
 								 3) / 186,
@@ -1115,34 +1235,40 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 	return 0;
 
     case 0x0a:			/* Error ID Data MID 10 */
-	return sirf_msg_errors(buf, len);
+	return sirf_msg_errors(buf, len, session->context->debug);
 
     case 0x0b:			/* Command Acknowledgement MID 11 */
-	gpsd_report(LOG_PROG, "SiRF: ACK 0x0b: %02x\n", getub(buf, 1));
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "SiRF: ACK 0x0b: %02x\n", getub(buf, 1));
+	session->driver.sirf.need_ack = false;
 	return 0;
 
     case 0x0c:			/* Command NAcknowledgement MID 12 */
-	gpsd_report(LOG_PROG, "SiRF: NAK 0x0c: %02x\n", getub(buf, 1));
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "SiRF: NAK 0x0c: %02x\n", getub(buf, 1));
+	/* ugh -- there's no alternative but silent failure here */
+	session->driver.sirf.need_ack = false;
 	return 0;
 
     case 0x0d:			/* Visible List MID 13 */
-	gpsd_report(LOG_PROG, "SiRF: unused VIS 0x0d\n");
+	gpsd_report(session->context->debug, LOG_PROG, "SiRF: unused VIS 0x0d\n");
 	return 0;
 
     case 0x0e:			/* Almanac Data MID 14 */
-	gpsd_report(LOG_PROG, "SiRF: unused ALM  0x0e\n");
+	gpsd_report(session->context->debug, LOG_PROG, "SiRF: unused ALM  0x0e\n");
 	return 0;
 
     case 0x0f:			/* Ephemeris Data MID 15 */
-	gpsd_report(LOG_PROG, "SiRF: unused EPH  0x0f\n");
+	gpsd_report(session->context->debug, LOG_PROG, "SiRF: unused EPH  0x0f\n");
 	return 0;
 
     case 0x11:			/* Differential Corrections MID 17 */
-	gpsd_report(LOG_PROG, "SiRF: unused DIFF 0x11\n");
+	gpsd_report(session->context->debug, LOG_PROG, "SiRF: unused DIFF 0x11\n");
 	return 0;
 
     case 0x12:			/* OK To Send MID 18 */
-	gpsd_report(LOG_PROG, "SiRF: OTS 0x12: send indicator = %d\n",
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "SiRF: OTS 0x12: send indicator = %d\n",
 		    getub(buf, 1));
 	return 0;
 
@@ -1153,27 +1279,26 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 	return sirf_msg_dgpsstatus(session, buf, len);
 
     case 0x1c:			/* Navigation Library Measurement Data MID 28 */
-	gpsd_report(LOG_PROG, "SiRF: NLMD 0x1c\n");
 	return sirf_msg_nlmd(session, buf, len);
 
     case 0x1d:			/* Navigation Library DGPS Data MID 29 */
-	gpsd_report(LOG_PROG, "SiRF: unused NLDG 0x1d\n");
+	gpsd_report(session->context->debug, LOG_PROG, "SiRF: unused NLDG 0x1d\n");
 	return 0;
 
     case 0x1e:			/* Navigation Library SV State Data MID 30 */
-	gpsd_report(LOG_PROG, "SiRF: unused NLSV 0x1e\n");
+	gpsd_report(session->context->debug, LOG_PROG, "SiRF: unused NLSV 0x1e\n");
 	return 0;
 
     case 0x1f:			/* Navigation Library Initialization Data MID 31 */
-	gpsd_report(LOG_PROG, "SiRF: unused NLID 0x1f\n");
+	gpsd_report(session->context->debug, LOG_PROG, "SiRF: unused NLID 0x1f\n");
 	return 0;
 
     case 0x29:			/* Geodetic Navigation Data MID 41 */
-	gpsd_report(LOG_PROG, "SiRF: unused GND 0x29\n");
+	gpsd_report(session->context->debug, LOG_PROG, "SiRF: unused GND 0x29\n");
 	return 0;
 
     case 0x32:			/* SBAS corrections MID 50 */
-	gpsd_report(LOG_PROG, "SiRF: unused SBAS 0x32\n");
+	gpsd_report(session->context->debug, LOG_PROG, "SiRF: unused SBAS 0x32\n");
 	return 0;
 
     case 0x34:			/* PPS Time MID 52 */
@@ -1200,22 +1325,47 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 	 */
 	return sirf_msg_ppstime(session, buf, len);
 
-    case 0x62:			/* uBlox Extended Measured Navigation Data MID 98 */
-	gpsd_report(LOG_PROG, "SiRF: uBlox EMND 0x62\n");
+    case 0x38:                /* EE Output MID 56 */
+	return sirf_msg_ee(session, buf, len);
+
+    case 0x40:                /* Nav Library MID 64 */
+	return sirf_msg_nl(session, buf, len);
+
+    case 0x47:                /* Hardware Config MID 71 */
+	gpsd_report(session->context->debug, LOG_PROG, 
+                    "SiRF IV: unused Hardware Config 0x47\n");
+	return 0;
+
+    case 0x5c:                /* CW Controller Output MID 92 */
+	gpsd_report(session->context->debug, LOG_PROG, 
+                    "SiRF IV: unused CW Controller Output 0x5c\n");
+	return 0;
+
+    case 0x5d:                /* TCXO Output MID 93 */
+	gpsd_report(session->context->debug, LOG_PROG, 
+                    "SiRF IV: unused TCXO Output 0x5d, SubID: %d\n",
+		    buf[1]);
+	return 0;
+
+    case 0x62:			/* u-blox Extended Measured Navigation Data MID 98 */
+	gpsd_report(session->context->debug, LOG_PROG, 
+                    "SiRF: u-blox EMND 0x62\n");
 	return sirf_msg_ublox(session, buf, len) | (CLEAR_IS | REPORT_IS);
 
     case 0x80:			/* Initialize Data Source MID 128 */
-	gpsd_report(LOG_PROG, "SiRF: unused INIT 0x80\n");
+	gpsd_report(session->context->debug, LOG_PROG, 
+                    "SiRF: unused INIT 0x80\n");
 	return 0;
 
     case 0xe1:			/* Development statistics messages MID 225 */
 	/* FALLTHROUGH */
     case 0xff:			/* Debug messages MID 255 */
-	(void)sirf_msg_debug(buf, len);
+	(void)sirf_msg_debug(buf, len, session->context->debug);
 	return 0;
 
     default:
-	gpsd_report(LOG_WARN, "SiRF: Unknown packet id %d length %zd\n",
+	gpsd_report(session->context->debug, LOG_PROG, 
+                    "SiRF: Unknown packet id %d length %zd\n",
 		    buf[0], len);
 	return 0;
     }
@@ -1223,18 +1373,12 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 
 static gps_mask_t sirfbin_parse_input(struct gps_device_t *session)
 {
-    gps_mask_t st;
-
     if (session->packet.type == SIRF_PACKET) {
-	st = sirf_parse(session, session->packet.outbuffer,
+	return sirf_parse(session, session->packet.outbuffer,
 			session->packet.outbuflen);
-	session->gpsdata.dev.driver_mode = MODE_BINARY;
-	return st;
 #ifdef NMEA_ENABLE
     } else if (session->packet.type == NMEA_PACKET) {
-	st = nmea_parse((char *)session->packet.outbuffer, session);
-	session->gpsdata.dev.driver_mode = MODE_NMEA;
-	return st;
+	return nmea_parse((char *)session->packet.outbuffer, session);
 #endif /* NMEA_ENABLE */
     } else
 	return 0;
@@ -1247,75 +1391,94 @@ static void sirfbin_event_hook(struct gps_device_t *session, event_t event)
 
     if (event == event_identified || event == event_reactivate) {
 	if (session->packet.type == NMEA_PACKET) {
-	    gpsd_report(LOG_PROG, "SiRF: Switching chip mode to binary.\n");
+	    gpsd_report(session->context->debug, LOG_PROG,
+			"SiRF: Switching chip mode to binary.\n");
 	    (void)nmea_send(session,
 			    "$PSRF100,0,%d,8,1,0",
 			    session->gpsdata.dev.baudrate);
-	    (void)usleep(3330); /* guessed settling time */
 	}
+    }
 
-	gpsd_report(LOG_PROG, "SiRF: Probing for firmware version...\n");
-	(void)sirf_write(session, versionprobe);
-	(void)usleep(3330); /* guessed settling time */
+    if (event == event_configure) {
+	/* might not be time for the next init string yet */ 
+	if (time(NULL) - session->driver.sirf.last_send > SIRF_RETRY_TIME)
+	    session->driver.sirf.need_ack = false;
+	/* can also be false because ACK was received after last send */
+	if (session->driver.sirf.need_ack)
+	    return;
 
-	gpsd_report(LOG_PROG, "SiRF: Requesting navigation parameters...\n");
-	(void)sirf_write(session, navparams);
-	(void)usleep(3330); /* guessed settling time */
+	switch (session->driver.sirf.cfg_stage++) {
+	case 0:
+	    /* this slot used by event_identified */
+	    return;
 
+	case 1:
+	    gpsd_report(session->context->debug, LOG_PROG,
+			"SiRF: Probing for firmware version...\n");
+	    (void)sirf_write(session, versionprobe);
+	    break;
+
+	case 2:
+		gpsd_report(session->context->debug, LOG_PROG,
+			    "SiRF: Requesting navigation parameters...\n");
+	    (void)sirf_write(session, navparams);
+	    break;
+
+	case 3:
 #ifdef RECONFIGURE_ENABLE
-	gpsd_report(LOG_PROG, "SiRF: Requesting periodic ecef reports...\n");
-	(void)sirf_write(session, requestecef);
-	(void)usleep(3330); /* guessed settling time */
+	    /* unset MID 64 first since there is a flood of them */
+	    gpsd_report(session->context->debug, LOG_PROG, "SiRF: unset MID 64...\n");
+	    putbyte(unsetmidXX, 6, 0x40);
+	    (void)sirf_write(session, unsetmidXX);
+	    break;
 
-	gpsd_report(LOG_PROG, "SiRF: Requesting periodic tracker reports...\n");
-	(void)sirf_write(session, requesttracker);
-	(void)usleep(3330); /* guessed settling time */
+	case 4:
+	    gpsd_report(session->context->debug, LOG_PROG,
+			"SiRF: Requesting periodic ecef reports...\n");
+	    (void)sirf_write(session, requestecef);
+	    break;
 
-	gpsd_report(LOG_PROG, "SiRF: Setting DGPS control to use SBAS...\n");
-	(void)sirf_write(session, dgpscontrol);
-	(void)usleep(3330); /* guessed settling time */
+	case 5:
+	    gpsd_report(session->context->debug, LOG_PROG,
+			"SiRF: Requesting periodic tracker reports...\n");
+	    (void)sirf_write(session, requesttracker);
+	    break;
 
-	gpsd_report(LOG_PROG, "SiRF: Setting SBAS to auto/integrity mode...\n");
-	(void)sirf_write(session, sbasparams);
-	(void)usleep(3330); /* guessed settling time */
+	case 7:
+	    gpsd_report(session->context->debug, LOG_PROG,
+			"SiRF: Setting DGPS control to use SBAS...\n");
+	    (void)sirf_write(session, dgpscontrol);
+	    break;
 
-	gpsd_report(LOG_PROG, "SiRF: Enabling PPS message...\n");
-	(void)sirf_write(session, enablemid52);
-	(void)usleep(3330); /* guessed settling time */
+	case 8:
+	    gpsd_report(session->context->debug, LOG_PROG,
+			"SiRF: Setting SBAS to auto/integrity mode...\n");
+	    (void)sirf_write(session, sbasparams);
+	    break;
 
-	if (session->gpsdata.dev.baudrate >= 38400) {
-	    /* some USB devices are also too slow, no way to tell which ones */
-	    gpsd_report(LOG_PROG, "SiRF: Enabling subframe transmission...\n");
-	    (void)sirf_write(session, enablesubframe);
-	    (void)usleep(3330); /* guessed settling time */
-	}
+	case 9:
+	    gpsd_report(session->context->debug, LOG_PROG,
+			"SiRF: Enabling PPS message...\n");
+	    (void)sirf_write(session, enablemid52);
 
-	/* disable some MIDs. we do not decode it, so don't send it */
-	gpsd_report(LOG_PROG, "SiRF: unset MID 7...\n");
-	putbyte(unsetmidXX, 6, 0x11);
-	(void)sirf_write(session, unsetmidXX);
-	(void)usleep(3330); /* guessed settling time */
-
-	gpsd_report(LOG_PROG, "SiRF: unset MID 28...\n");
-	putbyte(unsetmidXX, 6, 0x1c);
-	(void)sirf_write(session, unsetmidXX);
-	(void)usleep(3330); /* guessed settling time */
-
-	gpsd_report(LOG_PROG, "SiRF: unset MID 29...\n");
-	putbyte(unsetmidXX, 6, 0x1d);
-	(void)sirf_write(session, unsetmidXX);
-	(void)usleep(3330); /* guessed settling time */
-
-	gpsd_report(LOG_PROG, "SiRF: unset MID 30...\n");
-	putbyte(unsetmidXX, 6, 0x1e);
-	(void)sirf_write(session, unsetmidXX);
-	(void)usleep(3330); /* guessed settling time */
-
-	gpsd_report(LOG_PROG, "SiRF: unset MID 31...\n");
-	putbyte(unsetmidXX, 6, 0x1f);
-	(void)sirf_write(session, unsetmidXX);
-	(void)usleep(3330); /* guessed settling time */
+	    /* SiRF recommends at least 57600 for SiRF IV nav data */
+	    if (session->gpsdata.dev.baudrate >= 57600) {
+		/* fast enough, turn on nav data */
+		gpsd_report(session->context->debug, LOG_PROG,
+			    "SiRF: Enabling subframe transmission...\n");
+		(void)sirf_write(session, enablesubframe);
+	    } else {
+		/* too slow, turn off nav data */
+		gpsd_report(session->context->debug, LOG_PROG,
+			    "SiRF: Disabling subframe transmission...\n");
+		(void)sirf_write(session, disablesubframe);
+	    }
+	    break;
 #endif /* RECONFIGURE_ENABLE */
+	default:
+	    /* initialization is done */
+	    return;
+	}
     }
 
     if (event == event_deactivate) {
@@ -1344,22 +1507,25 @@ static void sirfbin_event_hook(struct gps_device_t *session, event_t event)
 	putbyte(moderevert, 16, session->driver.sirf.dr_timeout);
 	putbyte(moderevert, 17, session->driver.sirf.track_smooth_mode);
 	/*@ +shiftimplementation @*/
-	gpsd_report(LOG_PROG, "SiRF: Reverting navigation parameters...\n");
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "SiRF: Reverting navigation parameters...\n");
 	(void)sirf_write(session, moderevert);
     }
 }
 
+
+
 /* this is everything we export */
 /* *INDENT-OFF* */
-const struct gps_type_t sirf_binary =
+const struct gps_type_t driver_sirf =
 {
-    .type_name      = "SiRF binary",	/* full name of type */
+    .type_name      = "SiRF",		/* full name of type */
     .packet_type    = SIRF_PACKET,	/* associated lexer packet type */
-    .flags	    = DRIVER_NOFLAGS,	/* no flags set */
+    .flags	    = DRIVER_STICKY,	/* no flags set */
     .trigger	    = NULL,		/* no trigger */
     .channels       = SIRF_CHANNELS,	/* consumer-grade GPS */
     .probe_detect   = NULL,		/* no probe */
-    .get_packet     = sirf_get,		/* be prepared for SiRF or NMEA */
+    .get_packet     = generic_get,	/* be prepared for SiRF or NMEA */
     .parse_packet   = sirfbin_parse_input,/* parse message packets */
     .rtcm_writer    = gpsd_write,	/* send RTCM data straight */
     .event_hook     = sirfbin_event_hook,/* lifetime event handler */
@@ -1372,8 +1538,8 @@ const struct gps_type_t sirf_binary =
 #ifdef CONTROLSEND_ENABLE
     .control_send   = sirf_control_send,/* how to send a control string */
 #endif /* CONTROLSEND_ENABLE */
-#ifdef NTPSHM_ENABLE
-    .ntp_offset     = sirf_ntp_offset,
+#ifdef TIMEHINT_ENABLE
+    .time_offset     = sirf_time_offset,
 #endif /* NTP_SHM_ENABLE */
 };
 /* *INDENT-ON* */
