@@ -1,6 +1,7 @@
 /*
  * Handle the Trimble TSIP packet format
  * by Rob Janssen, PE1CHL.
+ * Accutime Gold support by Igor Socec <igorsocec@gmail.com>
  *
  * Week counters are not limited to 10 bits. It's unknown what
  * the firmware is doing to disambiguate them, if anything; it might just
@@ -27,6 +28,9 @@
 
 #define SEMI_2_DEG	(180.0 / 2147483647)	/* 2^-31 semicircle to deg */
 
+void configuration_packets_accutime_gold(struct gps_device_t *session);
+void configuration_packets_generic(struct gps_device_t *session);
+
 #ifdef TSIP_ENABLE
 #define TSIP_CHANNELS	12
 
@@ -51,7 +55,8 @@ static int tsip_write(struct gps_device_t *session,
     *ep++ = '\x03';
     session->msgbuflen = (size_t) (ep - session->msgbuf);
     /*@ -charint @*/
-    gpsd_report(LOG_IO, "Sent TSIP packet id 0x%02x\n", id);
+    gpsd_report(session->context->debug, LOG_PROG,
+		"Sent TSIP packet id 0x%02x\n", id);
     if (gpsd_write(session, session->msgbuf, session->msgbuflen) !=
 	(ssize_t) session->msgbuflen)
 	return -1;
@@ -99,7 +104,8 @@ static bool tsip_detect(struct gps_device_t *session)
 		break;
 	    if (generic_get(session) >= 0) {
 		if (session->packet.type == TSIP_PACKET) {
-		    gpsd_report(LOG_RAW, "tsip_detect found\n");
+		    gpsd_report(session->context->debug, LOG_RAW,
+				"tsip_detect found\n");
 		    ret = true;
 		    break;
 		}
@@ -114,12 +120,12 @@ static bool tsip_detect(struct gps_device_t *session)
     return ret;
 }
 
-static gps_mask_t tsip_analyze(struct gps_device_t *session)
+static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 {
     int i, j, len, count;
     gps_mask_t mask = 0;
     unsigned int id;
-    uint8_t u1, u2, u3, u4, u5;
+    uint8_t u1, u2, u3, u4, u5, u6, u7;
     int16_t s1, s2, s3, s4;
     int32_t sl1, sl2, sl3;
     uint32_t ul1, ul2;
@@ -130,7 +136,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
     char buf2[BUFSIZ];
 
     if (session->packet.type != TSIP_PACKET) {
-	gpsd_report(LOG_INF, "tsip_analyze packet type %d\n",
+	gpsd_report(session->context->debug, LOG_INF, "tsip_analyze packet type %d\n",
 		    session->packet.type);
 	return 0;
     }
@@ -157,8 +163,9 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
     (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag),
 		   "ID%02x", id = (unsigned)session->packet.outbuffer[1]);
 
-    gpsd_report(LOG_IO, "TSIP packet id 0x%02x length %d: %s\n", id, len,
-		buf2);
+    gpsd_report(session->context->debug, LOG_DATA,
+		"TSIP packet id 0x%02x length %d: %s\n",
+		id, len, buf2);
     (void)time(&now);
 
     session->cycle_end_reliable = true;
@@ -166,11 +173,12 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
     case 0x13:			/* Packet Received */
 	u1 = getub(buf, 0);
 	u2 = getub(buf, 1);
-	gpsd_report(LOG_WARN,
+	gpsd_report(session->context->debug, LOG_WARN,
 		    "Received packet of type %02x cannot be parsed\n", u1);
 #if USE_SUPERPACKET
 	if ((int)u1 == 0x8e && (int)u2 == 0x23) {	/* no Compact Super Packet */
-	    gpsd_report(LOG_WARN, "No Compact Super Packet, use LFwEI\n");
+	    gpsd_report(session->context->debug, LOG_WARN,
+			"No Compact Super Packet, use LFwEI\n");
 
 	    /* Request LFwEI Super Packet */
 	    putbyte(buf, 0, 0x20);
@@ -178,6 +186,78 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    (void)tsip_write(session, 0x8e, buf, 2);
 	}
 #endif /* USE_SUPERPACKET */
+
+	break;
+    case 0x1c: /* Hardware/Software Version Information (Accutime Gold) */
+	/*
+	 * FIXME: We could get both ginds of version info.
+	 */
+	u1 = (uint8_t) getub(buf, 0); 
+	if (u1 == 0x81) { /* Software Version Information */
+		(void)snprintf(session->gpsdata.tag + strlen(session->gpsdata.tag), 
+			sizeof(session->gpsdata.tag) -
+			strlen(session->gpsdata.tag), "%02x", (uint) u1);
+		u2 = getub(buf, 2); /* Major version */
+		u3 = getub(buf, 3); /* Minor version */
+		u4 = getub(buf, 4); /* Build number */
+		u5 = getub(buf, 5); /* Month */
+		u6 = getub(buf, 6); /* Day */
+		s1 = (int16_t)getbeu16(buf, 7); /* Year */
+		u7 = getub(buf, 9); /* Length of first module name */
+		for (i=0; i < (int)u7; i++) {
+		    buf2[i] = (char)getub(buf, 10+i); /* Product name in ASCII */
+		}
+		buf2[i] = '\0';
+
+		/*@ -formattype @*/
+		(void)snprintf(session->subtype, sizeof(session->subtype),
+			       "sw %u %u %u %02u.%02u.%04u %s",
+			       u2, u3, u4, u6, u5, s1, buf2);
+		/*@ +formattype @*/
+		gpsd_report(session->context->debug, LOG_INF,
+			    "Software version: %s\n", 
+			    session->subtype);
+
+		mask |= DEVICEID_SET;
+	}
+	if (u1 == 0x83) { /* Hardware Version Information */
+		(void)snprintf(session->gpsdata.tag + strlen(session->gpsdata.tag), 
+			sizeof(session->gpsdata.tag) -
+			strlen(session->gpsdata.tag), "%02x", (uint) u1);
+		ul1 = getbeu32(buf, 1); /* Serial number */
+		u2 = getub(buf, 5); /* Build day */
+		u3 = getub(buf, 6); /* Build month */
+		s1 = (int16_t)getbeu16(buf, 7); /* Build year */
+		u4 = getub(buf, 6); /* Build hour */
+		s2 = (int16_t)getbeu16(buf, 10); /* Hardware Code */
+		u5 = getub(buf, 12); /* Length of Hardware ID */
+		for (i=0; i < (int)u5; i++) {
+		    buf2[i] = (char)getub(buf, 13+i); /* Hardware ID in ASCII */
+		}
+		buf2[i] = '\0';
+
+		/*@ -formattype @*/
+		(void)snprintf(session->subtype, sizeof(session->subtype),
+			       "hw %u %02u.%02u.%04u %02u %u %s",
+			       ul1, u2, u3, s1, u4, s2, buf2);
+		gpsd_report(session->context->debug, LOG_INF,
+			    "Hardware version: %s\n", 
+			    session->subtype);
+
+		mask |= DEVICEID_SET;
+
+		/* Detecting device by Hardware Code */
+		if (s2 == 3001) {
+			gpsd_report(session->context->debug, LOG_INF,
+				    "This device is Accutime Gold\n");
+			session->driver.tsip.subtype = TSIP_ACCUTIME_GOLD;
+			configuration_packets_accutime_gold(session);
+		}
+		else {
+			configuration_packets_generic(session);
+		}
+		break;
+	}
 	break;
     case 0x41:			/* GPS Time */
 	if (len != 10)
@@ -193,7 +273,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 		gpsd_gpstime_resolve(session, (unsigned short)s1, (double)f1);
 	    mask |= TIME_SET | PPSTIME_IS;
 	}
-	gpsd_report(LOG_INF, "GPS Time %f %d %f\n", f1, s1, f2);
+	gpsd_report(session->context->debug, LOG_INF,
+		    "GPS Time %f %d %f\n", f1, s1, f2);
 	break;
     case 0x42:			/* Single-Precision Position Fix, XYZ ECEF */
 	if (len != 16)
@@ -202,8 +283,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	f2 = getbef32((char *)buf, 4);	/* Y */
 	f3 = getbef32((char *)buf, 8);	/* Z */
 	f4 = getbef32((char *)buf, 12);	/* time-of-fix */
-	gpsd_report(LOG_INF, "GPS Position XYZ %f %f %f %f\n", f1, f2, f3,
-		    f4);
+	gpsd_report(session->context->debug, LOG_INF,
+		    "GPS Position XYZ %f %f %f %f\n", f1, f2, f3, f4);
 	break;
     case 0x43:			/* Velocity Fix, XYZ ECEF */
 	if (len != 20)
@@ -213,7 +294,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	f3 = getbef32((char *)buf, 8);	/* Z velocity */
 	f4 = getbef32((char *)buf, 12);	/* bias rate */
 	f5 = getbef32((char *)buf, 16);	/* time-of-fix */
-	gpsd_report(LOG_INF, "GPS Velocity XYZ %f %f %f %f %f\n", f1, f2, f3,
+	gpsd_report(session->context->debug, LOG_INF,
+		    "GPS Velocity XYZ %f %f %f %f %f\n", f1, f2, f3,
 		    f4, f5);
 	break;
     case 0x45:			/* Software Version Information */
@@ -222,13 +304,19 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	/*@ -formattype @*/
 	(void)snprintf(session->subtype, sizeof(session->subtype),
 		       "%d.%d %02d%02d%02d %d.%d %02d%02d%02d",
-		       getub(buf, 0), getub(buf, 1), getub(buf, 4), getub(buf,
-									  2),
-		       getub(buf, 3), getub(buf, 5), getub(buf, 6), getub(buf,
-									  9),
-		       getub(buf, 7), getub(buf, 8));
+		       getub(buf, 0), 
+		       getub(buf, 1), 
+		       getub(buf, 4), 
+		       getub(buf, 2),
+		       getub(buf, 3),
+		       getub(buf, 5),
+		       getub(buf, 6), 
+		       getub(buf, 9),
+		       getub(buf, 7), 
+		       getub(buf, 8));
 	/*@ +formattype @*/
-	gpsd_report(LOG_INF, "Software version: %s\n", session->subtype);
+	gpsd_report(session->context->debug, LOG_INF,
+		    "Software version: %s\n", session->subtype);
 	mask |= DEVICEID_SET;
 	break;
     case 0x46:			/* Health of Receiver */
@@ -246,7 +334,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 		mask |= STATUS_SET;
 	    }
 	}
-	gpsd_report(LOG_PROG, "Receiver health %02x %02x\n", u1, u2);
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "Receiver health %02x %02x\n", u1, u2);
 	break;
     case 0x47:			/* Signal Levels for all Satellites */
 	gpsd_zero_satellites(&session->gpsdata);
@@ -266,12 +355,14 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    (void)snprintf(buf2 + strlen(buf2), sizeof(buf2) - strlen(buf2),
 			   " %d=%.1f", (int)u1, f1);
 	}
-	gpsd_report(LOG_PROG, "Signal Levels (%d):%s\n", count, buf2);
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "Signal Levels (%d):%s\n", count, buf2);
 	mask |= SATELLITE_SET;
 	break;
     case 0x48:			/* GPS System Message */
 	buf[len] = '\0';
-	gpsd_report(LOG_PROG, "GPS System Message: %s\n", buf);
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "GPS System Message: %s\n", buf);
 	break;
     case 0x49:			/* Almanac Health Page */
 	break;
@@ -291,8 +382,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    mask |= TIME_SET | PPSTIME_IS;
 	}
 	mask |= LATLON_SET | ALTITUDE_SET | CLEAR_IS | REPORT_IS;
-	gpsd_report(LOG_DATA, "SPPLLA 0x4a "
-		    "time=%.2f lat=%.2f lon=%.2f alt=%.2f\n",
+	gpsd_report(session->context->debug, LOG_DATA, 
+		    "SPPLLA 0x4a time=%.2f lat=%.2f lon=%.2f alt=%.2f\n",
 		    session->newdata.time,
 		    session->newdata.latitude,
 		    session->newdata.longitude,
@@ -304,10 +395,12 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	u1 = getub(buf, 0);	/* Machine ID */
 	u2 = getub(buf, 1);	/* Status 1 */
 	u3 = getub(buf, 2);	/* Status 2 */
-	gpsd_report(LOG_INF, "Machine ID %02x %02x %02x\n", u1, u2, u3);
+	gpsd_report(session->context->debug, LOG_INF,
+		    "Machine ID %02x %02x %02x\n", u1, u2, u3);
 #if USE_SUPERPACKET
 	if ((u3 & 0x01) != (uint8_t) 0 && !session->driver.tsip.superpkt) {
-	    gpsd_report(LOG_PROG, "Switching to Super Packet mode\n");
+	    gpsd_report(session->context->debug, LOG_PROG,
+			"Switching to Super Packet mode\n");
 
 	    /* set new I/O Options for Super Packet output */
 	    putbyte(buf, 0, 0x2c);	/* Position: SP, MSL */
@@ -330,8 +423,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	u2 = getub(buf, 1);	/* Velocity */
 	u3 = getub(buf, 2);	/* Timing */
 	u4 = getub(buf, 3);	/* Aux */
-	gpsd_report(LOG_INF, "IO Options %02x %02x %02x %02x\n", u1, u2, u3,
-		    u4);
+	gpsd_report(session->context->debug, LOG_INF,
+		    "IO Options %02x %02x %02x %02x\n", u1, u2, u3, u4);
 #if USE_SUPERPACKET
 	if ((u1 & 0x20) != (uint8_t) 0) {	/* Output Super Packets? */
 	    /* No LFwEI Super Packet */
@@ -361,11 +454,12 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	/*@ +evalorder @*/
 	if ((session->newdata.track = atan2(f1, f2) * RAD_2_DEG) < 0)
 	    session->newdata.track += 360.0;
-	gpsd_report(LOG_INF, "GPS Velocity ENU %f %f %f %f %f\n", f1, f2, f3,
+	gpsd_report(session->context->debug, LOG_INF,
+		    "GPS Velocity ENU %f %f %f %f %f\n", f1, f2, f3,
 		    f4, f5);
 	mask |= SPEED_SET | TRACK_SET | CLIMB_SET;
-	gpsd_report(LOG_DATA, "VFENU 0x56 "
-		    "time=%.2f speed=%.2f track=%.2f climb=%.2f\n",
+	gpsd_report(session->context->debug, LOG_DATA, 
+		    "VFENU 0x56 time=%.2f speed=%.2f track=%.2f climb=%.2f\n",
 		    session->newdata.time,
 		    session->newdata.speed,
 		    session->newdata.track,
@@ -382,7 +476,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	if (getub(buf, 0) == 0x01)	/* good current fix? */
 	    (void)gpsd_gpstime_resolve(session, (unsigned short)s1, (double)f1);
 	/*@ -charint @*/
-	gpsd_report(LOG_INF, "Fix info %02x %02x %d %f\n", u1, u2, s1, f1);
+	gpsd_report(session->context->debug, LOG_INF,
+		    "Fix info %02x %02x %d %f\n", u1, u2, s1, f1);
 	break;
     case 0x58:			/* Satellite System Data/Acknowledge from Receiver */
 	break;
@@ -395,7 +490,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	f2 = getbef32((char *)buf, 9);	/* Code phase */
 	f3 = getbef32((char *)buf, 13);	/* Doppler */
 	d1 = getbed64((char *)buf, 17);	/* Time of Measurement */
-	gpsd_report(LOG_PROG, "Raw Measurement Data %d %f %f %f %f\n",
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "Raw Measurement Data %d %f %f %f %f\n",
 		    getub(buf, 0), f1, f2, f3, d1);
 	break;
     case 0x5b:			/* Satellite Ephemeris Status */
@@ -412,7 +508,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	d1 = getbef32((char *)buf, 12) * RAD_2_DEG;	/* Elevation */
 	d2 = getbef32((char *)buf, 16) * RAD_2_DEG;	/* Azimuth */
 	i = (int)(u2 >> 3);	/* channel number */
-	gpsd_report(LOG_INF,
+	gpsd_report(session->context->debug, LOG_INF,
 		    "Satellite Tracking Status: Ch %2d PRN %3d Res %d Acq %d Eph %2d SNR %4.1f LMT %.04f El %4.1f Az %5.1f\n",
 		    i, u1, u2 & 7, u3, u4, f1, f2, d1, d2);
 	if (i < TSIP_CHANNELS) {
@@ -486,8 +582,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 			   " %d", session->gpsdata.used[i] =
 			   (int)getub(buf, 17 + i));
 	/*@ -charint @*/
-	gpsd_report(LOG_DATA, "AIVSS: 0x6d "
-		    "status=%d used=%d "
+	gpsd_report(session->context->debug, LOG_DATA,
+		    "AIVSS: 0x6d status=%d used=%d "
 		    "pdop=%.1f hdop=%.1f vdop=%.1f tdop=%.1f gdup=%.1f\n",
 		    session->gpsdata.status,
 		    session->gpsdata.satellites_used,
@@ -524,7 +620,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    mask |= STATUS_SET;
 	}
 	/*@ -charint @*/
-	gpsd_report(LOG_DATA, "DPFM 0x82 status=%d\n", session->gpsdata.status);
+	gpsd_report(session->context->debug, LOG_DATA,
+		    "DPFM 0x82 status=%d\n", session->gpsdata.status);
 	break;
     case 0x83:			/* Double-Precision XYZ Position Fix and Bias Information */
 	if (len != 36)
@@ -534,8 +631,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	d3 = getbed64((char *)buf, 16);	/* Z */
 	d4 = getbed64((char *)buf, 24);	/* clock bias */
 	f1 = getbef32((char *)buf, 32);	/* time-of-fix */
-	gpsd_report(LOG_INF, "GPS Position XYZ %f %f %f %f %f\n", d1, d2, d3,
-		    d4, f1);
+	gpsd_report(session->context->debug, LOG_INF,
+		    "GPS Position XYZ %f %f %f %f %f\n", d1, d2, d3, d4, f1);
 	break;
     case 0x84:			/* Double-Precision LLA Position Fix and Bias Information */
 	if (len != 36)
@@ -552,13 +649,14 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 				  (double)f1);
 	    mask |= TIME_SET | PPSTIME_IS;
 	}
-	gpsd_report(LOG_INF, "GPS DP LLA %f %f %f %f\n",
+	gpsd_report(session->context->debug, LOG_INF,
+		    "GPS DP LLA %f %f %f %f\n",
 		    session->newdata.time,
 		    session->newdata.latitude,
 		    session->newdata.longitude, session->newdata.altitude);
 	mask |= LATLON_SET | ALTITUDE_SET | CLEAR_IS | REPORT_IS;
-	gpsd_report(LOG_DATA, "DPPLLA 0x84 "
-		    "time=%.2f lat=%.2f lon=%.2f alt=%.2f\n",
+	gpsd_report(session->context->debug, LOG_DATA,
+		    "DPPLLA 0x84 time=%.2f lat=%.2f lon=%.2f alt=%.2f\n",
 		    session->newdata.time,
 		    session->newdata.latitude,
 		    session->newdata.longitude,
@@ -581,7 +679,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    d3 = getbed64((char *)buf, 19);	/* DZ */
 	    d4 = getbed64((char *)buf, 27);	/* A-axis */
 	    d5 = getbed64((char *)buf, 35);	/* Eccentricity Squared */
-	    gpsd_report(LOG_INF, "Current Datum %d %f %f %f %f %f\n", s1, d1,
+	    gpsd_report(session->context->debug, LOG_INF,
+			"Current Datum %d %f %f %f %f %f\n", s1, d1,
 			d2, d3, d4, d5);
 	    break;
 
@@ -602,7 +701,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    u4 = getub(buf, 29);	/* utc offset */
 	    s4 = getbes16(buf, 30);	/* tsip.gps_week */
 	    /* PRN/IODE data follows */
-	    gpsd_report(LOG_RAW,
+	    gpsd_report(session->context->debug, LOG_RAW,
 			"LFwEI %d %d %d %u %d %u %u %x %x %u %u %d\n", s1, s2,
 			s3, ul1, sl1, ul2, sl2, u1, u2, u3, u4, s4);
 
@@ -650,7 +749,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 		TIME_SET | PPSTIME_IS | LATLON_SET | ALTITUDE_SET | SPEED_SET |
 		TRACK_SET | CLIMB_SET | STATUS_SET | MODE_SET | CLEAR_IS |
 		REPORT_IS;
-	    gpsd_report(LOG_DATA,
+	    gpsd_report(session->context->debug, LOG_DATA,
 			"SP-LFEI 0x20: time=%.2f lat=%.2f lon=%.2f alt=%.2f "
 			"speed=%.2f track=%.2f climb=%.2f "
 			"mode=%d status=%d\n",
@@ -675,7 +774,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    s2 = getbes16(buf, 21);	/* east velocity */
 	    s3 = getbes16(buf, 23);	/* north velocity */
 	    s4 = getbes16(buf, 25);	/* up velocity */
-	    gpsd_report(LOG_INF, "CSP %u %d %u %u %d %u %d %d %d %d\n", ul1,
+	    gpsd_report(session->context->debug, LOG_INF, "CSP %u %d %u %u %d %u %d %d %d %d\n", ul1,
 			s1, u1, u2, sl1, ul2, sl3, s2, s3, s4);
 	    if ((int)u1 > 10) {
 		session->context->leap_seconds = (int)u1;
@@ -721,7 +820,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 		TIME_SET | PPSTIME_IS | LATLON_SET | ALTITUDE_SET | SPEED_SET |
 		TRACK_SET |CLIMB_SET | STATUS_SET | MODE_SET | CLEAR_IS |
 		REPORT_IS;
-	    gpsd_report(LOG_DATA,
+	    gpsd_report(session->context->debug, LOG_DATA,
 			"SP-CSP 0x23: time=%.2f lat=%.2f lon=%.2f alt=%.2f "
 			"speed=%.2f track=%.2f climb=%.2f mode=%d status=%d\n",
 			session->newdata.time,
@@ -733,7 +832,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 
 	case 0xab:		/* Thunderbolt Timing Superpacket */
 	    if (len != 17) {
-		gpsd_report(4, "pkt 0xab len=%d\n", len);
+		gpsd_report(session->context->debug, 4, "pkt 0xab len=%d\n", len);
 		break;
 	    }
 	    session->driver.tsip.last_41 = now;	/* keep timestamp for request */
@@ -741,23 +840,25 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    s1 = (int16_t)getbeu16(buf, 5);	/* week */
 	    s2 = getbes16(buf, 7);	/* leap seconds */
 
-	    if ((int)u1 > 10) {
+	    if ((int)ul1 > 10) {
 		session->context->leap_seconds = (int)s2;
 		session->context->valid |= LEAP_SECOND_VALID;
 		session->newdata.time =
 		    gpsd_gpstime_resolve(session, (unsigned short)s1, (double)ul1);
 		mask |= TIME_SET | PPSTIME_IS | CLEAR_IS;
-		gpsd_report(LOG_DATA, "SP-TTS 0xab time=%.2f mask={TIME}\n",
+		gpsd_report(session->context->debug, LOG_DATA,
+			    "SP-TTS 0xab time=%.2f mask={TIME}\n",
 			    session->newdata.time);
 	    }
 
-	    gpsd_report(4, "GPS Time %u %d %d\n", ul1, s1, s2);
+	    gpsd_report(session->context->debug, 4,
+			"GPS Time %u %d %d\n", ul1, s1, s2);
 	    break;
 
 
 	case 0xac:		/* Thunderbolt Position Superpacket */
 	    if (len != 68) {
-		gpsd_report(4, "pkt 0xac len=%d\n", len);
+		gpsd_report(session->context->debug, 4, "pkt 0xac len=%d\n", len);
 
 		break;
 	    }
@@ -823,8 +924,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    }
 
 	    mask |= LATLON_SET | ALTITUDE_SET | MODE_SET | REPORT_IS;
-	    gpsd_report(LOG_DATA, "SP-TPS 0xac "
-			"time=%.2f lat=%.2f lon=%.2f alt=%.2f\n",
+	    gpsd_report(session->context->debug, LOG_DATA, 
+			"SP-TPS 0xac time=%.2f lat=%.2f lon=%.2f alt=%.2f\n",
 			session->newdata.time,
 			session->newdata.latitude,
 			session->newdata.longitude,
@@ -832,28 +933,29 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    break;
 
 	default:
-	    gpsd_report(LOG_WARN, "Unhandled TSIP superpacket type 0x%02x\n",
+	    gpsd_report(session->context->debug, LOG_WARN,
+			"Unhandled TSIP superpacket type 0x%02x\n",
 			u1);
 	}
 	break;
     case 0xbb:			/* Navigation Configuration */
-	if (len != 40)
+	if (len != 40 && len != 43)	/* see packet.c for explamation */
 	    break;
 	u1 = getub(buf, 0);	/* Subcode */
 	u2 = getub(buf, 1);	/* Operating Dimension */
-	u3 = getub(buf, 2);	/* DGPS Mode */
+	u3 = getub(buf, 2);	/* DGPS Mode (not enabled in Accutime Gold) */
 	u4 = getub(buf, 3);	/* Dynamics Code */
 	f1 = getbef32((char *)buf, 5);	/* Elevation Mask */
 	f2 = getbef32((char *)buf, 9);	/* AMU Mask */
 	f3 = getbef32((char *)buf, 13);	/* DOP Mask */
 	f4 = getbef32((char *)buf, 17);	/* DOP Switch */
-	u5 = getub(buf, 21);	/* DGPS Age Limit */
-	gpsd_report(LOG_INF,
+	u5 = getub(buf, 21);	/* DGPS Age Limit (not in Accutime Gold) */
+	gpsd_report(session->context->debug, LOG_INF,
 		    "Navigation Configuration %u %u %u %u %f %f %f %f %u\n",
 		    u1, u2, u3, u4, f1, f2, f3, f4, u5);
 	break;
     default:
-	gpsd_report(LOG_WARN, "Unhandled TSIP packet type 0x%02x\n", id);
+	gpsd_report(session->context->debug, LOG_WARN, "Unhandled TSIP packet type 0x%02x\n", id);
 	break;
     }
 
@@ -895,7 +997,8 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	((now - session->driver.tsip.req_compact) > 5)) {
 	/* Compact Superpacket requested but no response */
 	session->driver.tsip.req_compact = 0;
-	gpsd_report(LOG_WARN, "No Compact Super Packet, use LFwEI\n");
+	gpsd_report(session->context->debug, LOG_WARN,
+		    "No Compact Super Packet, use LFwEI\n");
 
 	/* Request LFwEI Super Packet */
 	putbyte(buf, 0, 0x20);
@@ -907,40 +1010,6 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
     return mask;
 }
 
-static gps_mask_t tsip_parse_input(struct gps_device_t *session)
-{
-    gps_mask_t st;
-
-    if (session->packet.type == TSIP_PACKET) {
-	st = tsip_analyze(session);
-	session->gpsdata.dev.driver_mode = MODE_BINARY;
-	return st;
-#ifdef EVERMORE_ENABLE
-    } else if (session->packet.type == EVERMORE_PACKET) {
-	(void)gpsd_switch_driver(session, "EverMore binary");
-	st = evermore_parse(session, session->packet.outbuffer,
-			    session->packet.outbuflen);
-	session->gpsdata.dev.driver_mode = MODE_BINARY;
-	return st;
-#endif /* EVERMORE_ENABLE */
-#ifdef SIRF_ENABLE
-	/*
-	 * mrd reported that once every couple of weeks his SiRF was flipping
-	 * into Trimble binary mode and not recovering.  Damn Trimble for not
-	 * checksumming their packets, it makes false positives hard to reject.
-	 * This should enable the SiRF to recover.
-	 */
-    } else if (session->packet.type == SIRF_PACKET) {
-	(void)gpsd_switch_driver(session, "SiRF binary");
-	st = sirf_parse(session, session->packet.outbuffer,
-			session->packet.outbuflen);
-	session->gpsdata.dev.driver_mode = MODE_BINARY;
-	return st;
-#endif /* SIRF_ENABLE */
-    } else
-	return 0;
-}
-
 #ifdef CONTROLSEND_ENABLE
 static ssize_t tsip_control_send(struct gps_device_t *session,
 				 char *buf, size_t buflen)
@@ -956,68 +1025,39 @@ static void tsip_event_hook(struct gps_device_t *session, event_t event)
 {
     if (session->context->readonly)
 	return;
-    /* FIX-ME: Resending this might not be needed on reactivation */
-    if (event == event_identified || event == event_reactivate) {
+    if (event == event_identified) {
 	unsigned char buf[100];
-
-	/* I/O Options */
+	
+	/*
+	 * Set basic configuration, in case no hardware config resonse
+	 * comes back.
+	 */
 	putbyte(buf, 0, 0x1e);	/* Position: DP, MSL, LLA */
 	putbyte(buf, 1, 0x02);	/* Velocity: ENU */
 	putbyte(buf, 2, 0x00);	/* Time: GPS */
 	putbyte(buf, 3, 0x08);	/* Aux: dBHz */
 	(void)tsip_write(session, 0x35, buf, 4);
+
+	/* Request Hardware Version Information */
+	putbyte(buf, 0, 0x03); /* Subcode */
+	(void)tsip_write(session, 0x1c, buf, 1);
+	/* 
+	 * After HW information packet is received, a 
+	 * decision is made how to configure the device.
+	 */
     }
-    if (event == event_configure) {
-	unsigned char buf[100];
-
-	switch (session->packet.counter) {
-	case 0:
-	    /*
-	     * TSIP is ODD parity 1 stopbit, save original values and
-	     * change it Thunderbolts and Copernicus use
-	     * 8N1... which isn't exactly a good idea due to the
-	     * fragile wire format.  We must divine a clever
-	     * heuristic to decide if the parity change is required.
-	     */
-	    session->driver.tsip.parity = session->gpsdata.dev.parity;
-	    session->driver.tsip.stopbits =
-		(uint) session->gpsdata.dev.stopbits;
-	    // gpsd_set_speed(session, session->gpsdata.dev.baudrate, 'O', 1);
-	    break;
-
-	case 1:
-	    /*@ -shiftimplementation @*/
-	    /* Request Software Versions */
-	    (void)tsip_write(session, 0x1f, NULL, 0);
-	    /* Request Current Time */
-	    (void)tsip_write(session, 0x21, NULL, 0);
-	    /* Set Operating Parameters */
-	    /* - dynamic code: land */
-	    putbyte(buf, 0, 0x01);
-	    /* - elevation mask */
-	    putbef32((char *)buf, 1, 5.0 * DEG_2_RAD);
-	    /* - signal level mask */
-	    putbef32((char *)buf, 5, 06.0);
-	    /* - PDOP mask */
-	    putbef32((char *)buf, 9, 8.0);
-	    /* - PDOP switch */
-	    putbef32((char *)buf, 13, 6.0);
-	    /*@ +shiftimplementation @*/
-	    (void)tsip_write(session, 0x2c, buf, 17);
-	    /* Set Position Fix Mode (auto 2D/3D) */
-	    putbyte(buf, 0, 0x00);
-	    (void)tsip_write(session, 0x22, buf, 1);
-	    /* Request GPS Systems Message */
-	    (void)tsip_write(session, 0x28, NULL, 0);
-	    /* Request Current Datum Values */
-	    (void)tsip_write(session, 0x37, NULL, 0);
-	    putbyte(buf, 0, 0x15);
-	    (void)tsip_write(session, 0x8e, buf, 1);
-	    /* Request Navigation Configuration */
-	    putbyte(buf, 0, 0x03);
-	    (void)tsip_write(session, 0xbb, buf, 1);
-	    break;
-	}
+    if (event == event_configure && session->packet.counter == 0) {
+	/*
+	 * TSIP is ODD parity 1 stopbit, save original values and
+	 * change it Thunderbolts and Copernicus use
+	 * 8N1... which isn't exactly a good idea due to the
+	 * fragile wire format.  We must divine a clever
+	 * heuristic to decide if the parity change is required.
+	 */
+	session->driver.tsip.parity = session->gpsdata.dev.parity;
+	session->driver.tsip.stopbits =
+	    (uint) session->gpsdata.dev.stopbits;
+	// gpsd_set_speed(session, session->gpsdata.dev.baudrate, 'O', 1);
     }
     if (event == event_deactivate) {
 	/* restore saved parity and stopbits when leaving TSIP mode */
@@ -1106,26 +1146,119 @@ static void tsip_mode(struct gps_device_t *session, int mode)
 	;
 
     } else {
-	gpsd_report(LOG_ERROR, "unknown mode %i requested\n", mode);
+	gpsd_report(session->context->debug, LOG_ERROR,
+		    "unknown mode %i requested\n", mode);
     }
 }
 #endif /* RECONFIGURE_ENABLE */
 
-#ifdef NTPSHM_ENABLE
-static double tsip_ntp_offset(struct gps_device_t *session UNUSED)
+#ifdef TIMEHINT_ENABLE
+static double tsip_time_offset(struct gps_device_t *session UNUSED)
 {
     /* FIX-ME: is a constant offset right here? */
     return 0.075;
 }
-#endif /* NTPSHM_ENABLE */
+#endif /* TIMEHINT_ENABLE */
+
+void configuration_packets_generic(struct gps_device_t *session)
+/* configure generic Trimble TSIP device to a known state */
+{
+	unsigned char buf[100];
+	
+	/* I/O Options */
+	putbyte(buf, 0, 0x1e);	/* Position: DP, MSL, LLA */
+	putbyte(buf, 1, 0x02);	/* Velocity: ENU */
+	putbyte(buf, 2, 0x00);	/* Time: GPS */
+	putbyte(buf, 3, 0x08);	/* Aux: dBHz */
+	(void)tsip_write(session, 0x35, buf, 4);
+	
+	/* Request Software Versions */
+	(void)tsip_write(session, 0x1f, NULL, 0);
+	/* Request Current Time */
+	(void)tsip_write(session, 0x21, NULL, 0);
+	/* Set Operating Parameters */
+	/* - dynamic code: land */
+	putbyte(buf, 0, 0x01);
+	/* - elevation mask */
+	putbef32((char *)buf, 1, (float)5.0 * DEG_2_RAD);
+	/* - signal level mask */
+	putbef32((char *)buf, 5, (float)06.0);
+	/* - PDOP mask */
+	putbef32((char *)buf, 9, (float)8.0);
+	/* - PDOP switch */
+	putbef32((char *)buf, 13, (float)6.0);
+	/*@ +shiftimplementation @*/
+	(void)tsip_write(session, 0x2c, buf, 17);
+	/* Set Position Fix Mode (auto 2D/3D) */
+	putbyte(buf, 0, 0x00);
+	(void)tsip_write(session, 0x22, buf, 1);
+	/* Request GPS Systems Message */
+	(void)tsip_write(session, 0x28, NULL, 0);
+	/* Request Current Datum Values */
+	(void)tsip_write(session, 0x37, NULL, 0);
+	putbyte(buf, 0, 0x15);
+	(void)tsip_write(session, 0x8e, buf, 1);
+	/* Request Navigation Configuration */
+	putbyte(buf, 0, 0x03);
+	(void)tsip_write(session, 0xbb, buf, 1);
+}
+
+void configuration_packets_accutime_gold(struct gps_device_t *session)
+/* configure Accutime Gold to a known state */
+{
+	unsigned char buf[100];
+	/* Request Software Version Information */
+	putbyte(buf, 0, 0x01);
+	(void)tsip_write(session, 0x1c, buf, 1);
+	/* Set Self-Survey Parameters */
+	putbyte(buf, 0, 0xa9); /* Subcode */
+	putbyte(buf, 1, 0x01); /* Self-Survey Enable = enable */
+	putbyte(buf, 2, 0x01); /* Position Save Flag = save position */
+	/*@-shiftimplementation@*/
+	putbe32(buf, 3, 2000); /* Self-Survey Length = 2000 */
+	putbe32(buf, 7, 0); /* Reserved */
+	/*@+shiftimplementation@*/
+	(void)tsip_write(session, 0x8e, buf, 11);
+	/* Set PPS Output Option */
+	putbyte(buf, 0, 0x4e); /* Subcode */
+	putbyte(buf, 1, 2); /* PPS driver switch = 2 (PPS is always output) */
+	(void)tsip_write(session, 0x8e, buf, 2);
+	/* Set Primary Receiver Configuration */
+	putbyte(buf, 0, 0x00); /* Subcode */
+	putbyte(buf, 1, 0x07); /* Operating Dimension = default full position 4, ODCM 7 */
+	putbyte(buf, 2, 0xff); /* Not enabled = unchanged */
+	putbyte(buf, 3, 0x01); /* Dynamics code = default */
+	putbyte(buf, 4, 0x01); /* Solution Mode = default */
+	putbef32((char *)buf, 5, (float)0.1745); /* Elevation Mask = 0.1745 */
+	putbef32((char *)buf, 9, (float)4.0); /* AMU Mask = default is 4.0 */
+	putbef32((char *)buf, 13, (float)8.0); /* PDOP Mask = 8.0 */
+	putbef32((char *)buf, 17, (float)6.0); /* PDOP Switch = 6.0 */
+	putbyte(buf, 21, 0xff); /* N/A */
+	putbyte(buf, 22, 0x0); /* Foliage Mode = default */
+	putbe16(buf, 23, 0xffff); /* Reserved */
+	putbe16(buf, 25, 0x0000); /* Measurement Rate and Position Fix Rate = default */
+	/*@-shiftimplementation@*/
+	putbe32(buf, 27, 0xffffffff); /* Reserved */
+	putbe32(buf, 31, 0xffffffff); /* Reserved */
+	putbe32(buf, 35, 0xffffffff); /* Reserved */
+	putbe32(buf, 39, 0xffffffff); /* Reserved */
+	/*@+shiftimplementation@*/
+	(void)tsip_write(session, 0xbb, buf, 43);
+	/* Set Packet Broadcast Mask */
+	putbyte(buf, 0, 0xa5); /* Subcode */
+	putbe16(buf, 1, 0x32e1); /* Packets bit field = default + Primary timing, Supplemental timing 32e1 */
+	putbyte(buf, 3, 0x00); /* not used */
+	putbyte(buf, 4, 0x00); /* not used */
+	(void)tsip_write(session, 0x8e, buf, 5);
+}
 
 /* this is everything we export */
 /* *INDENT-OFF* */
-const struct gps_type_t tsip_binary =
+const struct gps_type_t driver_tsip =
 {
     .type_name      = "Trimble TSIP",	/* full name of type */
     .packet_type    = TSIP_PACKET,	/* associated lexer packet type */
-    .flags	    = DRIVER_NOFLAGS,	/* no flags set */
+    .flags	    = DRIVER_STICKY,	/* remember this */
     .trigger	    = NULL,		/* no trigger */
     .channels       = TSIP_CHANNELS,	/* consumer-grade GPS */
     .probe_detect   = tsip_detect,	/* probe for 9600O81 device */
@@ -1142,9 +1275,9 @@ const struct gps_type_t tsip_binary =
 #ifdef CONTROLSEND_ENABLE
     .control_send   = tsip_control_send,/* how to send commands */
 #endif /* CONTROLSEND_ENABLE */
-#ifdef NTPSHM_ENABLE
-    .ntp_offset     = tsip_ntp_offset,
-#endif /* NTPSHM_ENABLE */
+#ifdef TIMEHINT_ENABLE
+    .time_offset     = tsip_time_offset,
+#endif /* TIMEHINT_ENABLE */
 };
 /* *INDENT-ON* */
 
