@@ -18,6 +18,7 @@
 #endif /* S_SPLINT_S */
 
 #include "gpsd.h"
+#include "bits.h"
 
 /* Zodiac protocol description uses 1-origin indexing by little-endian word */
 #define get16z(buf, n)	( (buf[2*(n)-2])	\
@@ -48,32 +49,22 @@ static unsigned short zodiac_checksum(unsigned short *w, int n)
     return -csum;
 }
 
-/* zodiac_spew - Takes a message type, an array of data words, and a length
-   for the array, and prepends a 5 word header (including checksum).
-   The data words are expected to be checksummed */
-#if defined (WORDS_BIGENDIAN)
-/* data is assumed to contain len/2 unsigned short words
- * we change the endianness to little, when needed.
- */
-static int end_write(int fd, void *d, int len)
+static ssize_t end_write(int fd, void *d, size_t len)
+/* write an array of shorts in little-endian format */
 {
-    char buf[BUFSIZ];
-    char *p = buf;
-    char *data = (char *)d;
-    size_t n = (size_t) len;
+    unsigned char buf[BUFSIZ];
+    short *data = (short *)d;
+    size_t n = (size_t)(len/2);
 
-    while (n > 0) {
-	*p++ = *(data + 1);
-	*p++ = *data;
-	data += 2;
-	n -= 2;
-    }
-    return write(fd, buf, len);
+    for (n = 0; n < (size_t)(len/2); n++)
+	putle16(buf, n*2, data[n]); 
+    return write(fd, (char*)buf, len);
 }
-#else
-#define end_write write
-#endif /* WORDS_BIGENDIAN */
 
+/* zodiac_spew - Takes a message type, an array of data words, and a length
+ * for the array, and prepends a 5 word header (including checksum).
+ * The data words are expected to be checksummed.
+ */
 static ssize_t zodiac_spew(struct gps_device_t *session, unsigned short type,
 			   unsigned short *dat, int dlen)
 {
@@ -94,7 +85,8 @@ static ssize_t zodiac_spew(struct gps_device_t *session, unsigned short type,
 	if (end_write(session->gpsdata.gps_fd, &h, hlen) != (ssize_t) hlen ||
 	    end_write(session->gpsdata.gps_fd, dat,
 		      datlen) != (ssize_t) datlen) {
-	    gpsd_report(LOG_RAW, "Reconfigure write failed\n");
+	    gpsd_report(session->context->debug, LOG_RAW,
+			"Reconfigure write failed\n");
 	    return -1;
 	}
     }
@@ -106,7 +98,8 @@ static ssize_t zodiac_spew(struct gps_device_t *session, unsigned short type,
 	(void)snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
 		       " %04x", dat[i]);
 
-    gpsd_report(LOG_RAW, "Sent Zodiac packet: %s\n", buf);
+    gpsd_report(session->context->debug, LOG_RAW,
+		"Sent Zodiac packet: %s\n", buf);
 
     return 0;
 }
@@ -131,10 +124,8 @@ static void send_rtcm(struct gps_device_t *session,
 static ssize_t zodiac_send_rtcm(struct gps_device_t *session,
 				const char *rtcmbuf, size_t rtcmbytes)
 {
-    size_t len;
-
     while (rtcmbytes > 0) {
-	len = (size_t) (rtcmbytes > 64 ? 64 : rtcmbytes);
+	size_t len = (size_t) (rtcmbytes > 64 ? 64 : rtcmbytes);
 	send_rtcm(session, rtcmbuf, len);
 	rtcmbytes -= len;
 	rtcmbuf += len;
@@ -212,7 +203,7 @@ static gps_mask_t handle1000(struct gps_device_t *session)
     mask =
 	TIME_SET | PPSTIME_IS | LATLON_SET | ALTITUDE_SET | CLIMB_SET | SPEED_SET |
 	TRACK_SET | STATUS_SET | MODE_SET;
-    gpsd_report(LOG_DATA,
+    gpsd_report(session->context->debug, LOG_DATA,
 		"1000: time=%.2f lat=%.2f lon=%.2f alt=%.2f track=%.2f speed=%.2f climb=%.2f mode=%d status=%d\n",
 		session->newdata.time, session->newdata.latitude,
 		session->newdata.longitude, session->newdata.altitude,
@@ -225,7 +216,7 @@ static gps_mask_t handle1000(struct gps_device_t *session)
 static gps_mask_t handle1002(struct gps_device_t *session)
 /* satellite signal quality report */
 {
-    int i, j, status, prn;
+    int i, j;
 
     /* ticks                      = getzlong(6); */
     /* sequence                   = getzword(8); */
@@ -240,6 +231,7 @@ static gps_mask_t handle1002(struct gps_device_t *session)
     session->gpsdata.satellites_used = 0;
     memset(session->gpsdata.used, 0, sizeof(session->gpsdata.used));
     for (i = 0; i < ZODIAC_CHANNELS; i++) {
+	int status, prn;
 	/*@ -type @*/
 	session->driver.zodiac.Zv[i] = status = (int)getzword(15 + (3 * i));
 	session->driver.zodiac.Zs[i] = prn = (int)getzword(16 + (3 * i));
@@ -257,7 +249,8 @@ static gps_mask_t handle1002(struct gps_device_t *session)
     session->gpsdata.skyview_time = gpsd_gpstime_resolve(session,
 						      (unsigned short)gps_week,
 						      (double)gps_seconds);
-    gpsd_report(LOG_DATA, "1002: visible=%d used=%d mask={SATELLITE|USED}\n",
+    gpsd_report(session->context->debug, LOG_DATA,
+		"1002: visible=%d used=%d mask={SATELLITE|USED}\n",
 		session->gpsdata.satellites_visible,
 		session->gpsdata.satellites_used);
     return SATELLITE_SET | USED_IS;
@@ -302,7 +295,8 @@ static gps_mask_t handle1003(struct gps_device_t *session)
 	}
     }
     session->gpsdata.skyview_time = NAN;
-    gpsd_report(LOG_DATA, "NAVDOP: visible=%d gdop=%.2f pdop=%.2f "
+    gpsd_report(session->context->debug, LOG_DATA,
+		"NAVDOP: visible=%d gdop=%.2f pdop=%.2f "
 		"hdop=%.2f vdop=%.2f tdop=%.2f mask={SATELLITE|DOP}\n",
 		session->gpsdata.satellites_visible,
 		session->gpsdata.dop.gdop,
@@ -336,7 +330,8 @@ static gps_mask_t handle1011(struct gps_device_t *session)
      * The Zodiac is supposed to send one of these messages on startup.
      */
     getstringz(session->subtype, session->packet.outbuffer, 19, 28);	/* software version field */
-    gpsd_report(LOG_DATA, "1011: subtype=%s mask={DEVICEID}\n",
+    gpsd_report(session->context->debug, LOG_DATA,
+		"1011: subtype=%s mask={DEVICEID}\n",
 		session->subtype);
     return DEVICEID_SET;
 }
@@ -357,49 +352,18 @@ static void handle1108(struct gps_device_t *session)
 
 static gps_mask_t zodiac_analyze(struct gps_device_t *session)
 {
-    char buf[BUFSIZ];
-    int i;
     unsigned int id =
 	(unsigned int)((session->packet.outbuffer[3] << 8) |
 		       session->packet.outbuffer[2]);
-
-    if (session->packet.type != ZODIAC_PACKET) {
-	const struct gps_type_t **dp;
-	gpsd_report(LOG_PROG, "zodiac_analyze packet type %d\n",
-		    session->packet.type);
-	// Wrong packet type ?
-	// Maybe find a trigger just in case it's an Earthmate
-	gpsd_report(LOG_RAW + 4, "Is this a trigger: %s ?\n",
-		    (char *)session->packet.outbuffer);
-
-	for (dp = gpsd_drivers; *dp; dp++) {
-	    char *trigger = (*dp)->trigger;
-
-	    if (trigger != NULL
-		&& strncmp((char *)session->packet.outbuffer, trigger,
-			   strlen(trigger)) == 0
-		&& isatty(session->gpsdata.gps_fd) != 0) {
-		gpsd_report(LOG_PROG, "found %s.\n", trigger);
-
-		(void)gpsd_switch_driver(session, (*dp)->type_name);
-		return 0;
-	    }
-	}
-	return 0;
-    }
-
-    buf[0] = '\0';
-    for (i = 0; i < (int)session->packet.outbuflen; i++)
-	(void)snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
-		       "%02x", (unsigned int)session->packet.outbuffer[i]);
-    gpsd_report(LOG_RAW, "Raw Zodiac packet type %d length %zd: %s\n",
-		id, session->packet.outbuflen, buf);
+    gpsd_report(session->context->debug, LOG_RAW,
+		"Raw Zodiac packet type %d length %zd: %s\n",
+		id, session->packet.outbuflen, gpsd_prettydump(session));
 
     if (session->packet.outbuflen < 10)
 	return 0;
 
-    (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag), "%u",
-		   id);
+    (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag), 
+		   "%u", id);
 
     /*
      * Normal cycle for these devices is 1001 1002.
@@ -480,8 +444,8 @@ static bool zodiac_speed_switch(struct gps_device_t *session,
 }
 #endif /* RECONFIGURE_ENABLE */
 
-#ifdef NTPSHM_ENABLE
-static double zodiac_ntp_offset(struct gps_device_t *session UNUSED)
+#ifdef TIMEHINT_ENABLE
+static double zodiac_time_offset(struct gps_device_t *session UNUSED)
 {
     /* Removing/changing the magic number below is likely to disturb
      * the handling of the 1pps signal from the gps device. The regression
@@ -489,15 +453,15 @@ static double zodiac_ntp_offset(struct gps_device_t *session UNUSED)
      * with the 1pps signal active is required. */
     return 1.1;
 }
-#endif /* NTPSHM_ENABLE */
+#endif /* TIMEHINT_ENABLE */
 
 /* this is everything we export */
 /* *INDENT-OFF* */
-const struct gps_type_t zodiac_binary =
+const struct gps_type_t driver_zodiac =
 {
-    .type_name      = "Zodiac binary",	/* full name of type */
+    .type_name      = "Zodiac",		/* full name of type */
     .packet_type    = ZODIAC_PACKET,	/* associated lexer packet type */
-    .flags	    = DRIVER_NOFLAGS,	/* no flags set */
+    .flags	    = DRIVER_STICKY,	/* no flags set */
     .trigger	    = NULL,		/* no trigger */
     .channels       = 12,		/* consumer-grade GPS */
     .probe_detect   = NULL,		/* no probe */
@@ -514,9 +478,9 @@ const struct gps_type_t zodiac_binary =
 #ifdef CONTROLSEND_ENABLE
     .control_send   = zodiac_control_send,	/* for gpsctl and friends */
 #endif /* CONTROLSEND_ENABLE */
-#ifdef NTPSHM_ENABLE
-    .ntp_offset     = zodiac_ntp_offset,	/* compute NTO fudge factor */
-#endif /* NTPSHM_ENABLE */
+#ifdef TIMEHINT_ENABLE
+    .time_offset     = zodiac_time_offset,	/* compute NTO fudge factor */
+#endif /* TIMEHINT_ENABLE */
 };
 /* *INDENT-ON* */
 
