@@ -20,14 +20,15 @@
 #include "gpsdclient.h"
 
 #ifdef NMEA_ENABLE
-extern const struct gps_type_t nmea;
+extern const struct gps_type_t driver_nmea0183;
 
 static WINDOW *cookedwin, *nmeawin, *satwin, *gprmcwin, *gpggawin, *gpgsawin, *gpgstwin;
 static timestamp_t last_tick, tick_interval;
+static char sentences[NMEA_MAX * 2];
 
 /*****************************************************************************
  *
- * Generic NMEA support
+ * NMEA0183 support
  *
  *****************************************************************************/
 
@@ -83,7 +84,7 @@ static bool nmea_initialize(void)
     (void)mvwprintw(gprmcwin, 8, 12, " RMC ");
     (void)wattrset(gprmcwin, A_NORMAL);
 
-    gpgsawin = derwin(devicewin, 5, 30, 15, 20);
+    gpgsawin = derwin(devicewin, 6, 30, 15, 20);
     assert(gpgsawin !=NULL);
     (void)wborder(gpgsawin, 0, 0, 0, 0, 0, 0, 0, 0);
     (void)syncok(gpgsawin, true);
@@ -91,7 +92,11 @@ static bool nmea_initialize(void)
     (void)mvwprintw(gpgsawin, 1, 1, "Mode: ");
     (void)mvwprintw(gpgsawin, 2, 1, "Sats: ");
     (void)mvwprintw(gpgsawin, 3, 1, "DOP: H=      V=      P=");
-    (void)mvwprintw(gpgsawin, 4, 12, " GSA ");
+    (void)mvwprintw(gpgsawin, 4, 1, "PPS offset: ");
+#ifndef PPS_ENABLE
+    (void)mvwaddstr(gpgsawin, 4, 13, "Not available");
+#endif /* PPS_ENABLE */
+    (void)mvwprintw(gpgsawin, 5, 9, " GSA + PPS ");
     (void)wattrset(gpgsawin, A_NORMAL);
 
     gpggawin = derwin(devicewin, 9, 30, 6, 50);
@@ -129,6 +134,8 @@ static bool nmea_initialize(void)
 
     last_tick = timestamp();
 
+    sentences[0] = '\0';
+
     return (nmeawin != NULL);
     /*@ +globstate @*/
 }
@@ -169,8 +176,10 @@ static void cooked_pvt(void)
 /*@ -globstate -nullpass (splint is confused) */
 static void nmea_update(void)
 {
-    static char sentences[NMEA_MAX * 2];
     char **fields;
+#ifdef PPS_ENABLE
+    struct timedrift_t drift;
+#endif /* PPS_ENABLE */
 
     assert(cookedwin != NULL);
     assert(nmeawin != NULL);
@@ -310,6 +319,18 @@ static void nmea_update(void)
 	    (void)mvwprintw(gpgstwin, 4, 21,  "%-8s", fields[8]);
 	}
     }
+
+#ifdef PPS_ENABLE
+    /*@-compdef@*/
+    /*@-type@*/ /* splint is confused about struct timespec */
+    if (pps_thread_lastpps(&session, &drift) > 0) {
+	double timedelta = timespec_diff_ns(drift.real, drift.clock) * 1e-9;
+	(void)mvwprintw(gpgsawin, 4, 13, "%.9f", timedelta);
+	(void)wnoutrefresh(gpgsawin);
+    }
+    /*@+type@*/
+    /*@+compdef@*/
+#endif /* PPS_ENABLE */
 }
 
 /*@ +globstate +nullpass */
@@ -330,7 +351,7 @@ const struct monitor_object_t nmea_mmt = {
     .command = NULL,
     .wrap = nmea_wrap,
     .min_y = 21,.min_x = 80,
-    .driver = &nmea,
+    .driver = &driver_nmea0183,
 };
 
 /*****************************************************************************
@@ -362,7 +383,7 @@ static void monitor_nmea_send(const char *fmt, ...)
  */
 
 #if defined(GARMIN_ENABLE) && defined(NMEA_ENABLE)
-extern const struct gps_type_t garmin;
+extern const struct gps_type_t driver_garmin;
 
 const struct monitor_object_t garmin_mmt = {
     .initialize = nmea_initialize,
@@ -370,12 +391,12 @@ const struct monitor_object_t garmin_mmt = {
     .command = NULL,
     .wrap = nmea_wrap,
     .min_y = 21,.min_x = 80,
-    .driver = &garmin,
+    .driver = &driver_garmin,
 };
 #endif /* GARMIN_ENABLE && NMEA_ENABLE */
 
 #ifdef ASHTECH_ENABLE
-extern const struct gps_type_t ashtech;
+extern const struct gps_type_t driver_ashtech;
 
 #define ASHTECH_SPEED_9600 5
 #define ASHTECH_SPEED_57600 8
@@ -440,12 +461,12 @@ const struct monitor_object_t ashtech_mmt = {
 #endif /* CONTROLSEND_ENABLE */
     .wrap = nmea_wrap,
     .min_y = 21,.min_x = 80,
-    .driver = &ashtech,
+    .driver = &driver_ashtech,
 };
 #endif /* ASHTECH_ENABLE */
 
 #ifdef FV18_ENABLE
-extern const struct gps_type_t fv18;
+extern const struct gps_type_t driver_fv18;
 
 const struct monitor_object_t fv18_mmt = {
     .initialize = nmea_initialize,
@@ -453,12 +474,12 @@ const struct monitor_object_t fv18_mmt = {
     .command = NULL,
     .wrap = nmea_wrap,
     .min_y = 21,.min_x = 80,
-    .driver = &fv18,
+    .driver = &driver_fv18,
 };
 #endif /* FV18_ENABLE */
 
 #ifdef GPSCLOCK_ENABLE
-extern const struct gps_type_t gpsclock;
+extern const struct gps_type_t driver_gpsclock;
 
 const struct monitor_object_t gpsclock_mmt = {
     .initialize = nmea_initialize,
@@ -466,12 +487,12 @@ const struct monitor_object_t gpsclock_mmt = {
     .command = NULL,
     .wrap = nmea_wrap,
     .min_y = 21,.min_x = 80,
-    .driver = &gpsclock,
+    .driver = &driver_gpsclock,
 };
 #endif /* GPSCLOCK_ENABLE */
 
 #ifdef MTK3301_ENABLE
-extern const struct gps_type_t mtk3301;
+extern const struct gps_type_t driver_mtk3301;
 
 const struct monitor_object_t mtk3301_mmt = {
     .initialize = nmea_initialize,
@@ -479,8 +500,20 @@ const struct monitor_object_t mtk3301_mmt = {
     .command = NULL,
     .wrap = nmea_wrap,
     .min_y = 21,.min_x = 80,
-    .driver = &mtk3301,
+    .driver = &driver_mtk3301,
 };
 #endif /* MTK3301_ENABLE */
 
+#ifdef AIVDM_ENABLE
+extern const struct gps_type_t driver_aivdm;
+
+const struct monitor_object_t aivdm_mmt = {
+    .initialize = nmea_initialize,
+    .update = nmea_update,
+    .command = NULL,
+    .wrap = nmea_wrap,
+    .min_y = 21,.min_x = 80,
+    .driver = &driver_aivdm,
+};
+#endif /* AIVDM_ENABLE */
 #endif /* NMEA_ENABLE */
